// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:math';
import 'package:nertc_core_platform_interface/nertc_core_platform_interface.dart';
import 'package:nertc_core_platform_interface/pigeon.dart';

class MethodChannelNERtcEngine extends NERtcEnginePlatform {
  final _api = EngineApi();
  final _eventDispatcher = MethodChannelRtcChannelEventSink();

  MethodChannelNERtcEngine() {
    NERtcChannelEventSink.setup(_eventDispatcher);
  }

  void addRtcChannelListener(NERtcChannelEventSink sink) {
    _eventDispatcher.addRtcChannelListener(sink);
  }

  void removeRtcChannelListener(NERtcChannelEventSink sink) {
    _eventDispatcher.removeRtcChannelListener(sink);
  }

  @override
  Future<int> create(String appKey, NERtcOptions? options) async {
    var serverAddresses = RtcServerAddresses();
    final nertcServerAddresses = options?.serverAddresses;
    if (nertcServerAddresses != null) {
      serverAddresses
        ..valid = true
        ..channelServer = nertcServerAddresses.channelServer
        ..statisticsServer = nertcServerAddresses.statisticsServer
        ..roomServer = nertcServerAddresses.roomServer
        ..compatServer = nertcServerAddresses.compatServer
        ..nosLbsServer = nertcServerAddresses.nosLbsServer
        ..nosUploadSever = nertcServerAddresses.nosUploadSever
        ..nosTokenServer = nertcServerAddresses.nosTokenServer
        ..useIPv6 = nertcServerAddresses.useIPv6
        ..sdkConfigServer = nertcServerAddresses.sdkConfigServer
        ..cloudProxyServer = nertcServerAddresses.cloudProxyServer
        ..webSocketProxyServer = nertcServerAddresses.webSocketProxyServer
        ..quicProxyServer = nertcServerAddresses.quicProxyServer
        ..mediaProxyServer = nertcServerAddresses.mediaProxyServer
        ..statisticsDispatchServer =
            nertcServerAddresses.statisticsDispatchServer
        ..statisticsBackupServer = nertcServerAddresses.statisticsBackupServer;
    }
    int reply = await _api.create(CreateEngineRequest()
      ..appKey = appKey
      ..logDir = options?.logDir
      ..logLevel = options?.logLevel
      ..serverAddresses = serverAddresses
      ..audioAutoSubscribe = options?.audioAutoSubscribe
      ..videoAutoSubscribe = options?.videoAutoSubscribe
      ..disableFirstJoinUserCreateChannel =
          options?.disableFirstJoinUserCreateChannel
      ..audioDisableOverrideSpeakerOnReceiver =
          options?.audioDisableOverrideSpeakerOnReceiver
      ..audioAINSEnabled = options?.audioAINSEnabled
      ..audioDisableSWAECOnHeadset = options?.audioDisableSWAECOnHeadset
      ..videoEncodeMode = options?.videoEncodeMode?.index
      ..videoDecodeMode = options?.videoDecodeMode?.index
      ..videoCaptureObserverEnabled = options?.videoCaptureObserverEnabled
      ..serverRecordAudio = options?.serverRecordAudio
      ..serverRecordVideo = options?.serverRecordVideo
      ..serverRecordMode = options?.serverRecordMode?.index
      ..serverRecordSpeaker = options?.serverRecordSpeaker
      ..publishSelfStream = options?.publishSelfStream
      ..videoSendMode = options?.videoSendMode?.index
      ..videoH265Enabled = options?.videoH265Enabled);
    return reply;
  }

  @override
  Future<int> release() async {
    int reply = await _api.release();
    return reply;
  }

  Future<NERtcVersion> version() async {
    return await _api.version();
  }

  Future<List<String?>?> checkPermission() async {
    return await _api.checkPermission();
  }

  Future<int> setParameters(Map<String, Object> params) async {
    int reply = await _api.setParameters(params);
    return reply;
  }

  Future<int> setStatsEventCallback() async {
    int reply = await _api.setStatsEventCallback();
    return reply;
  }

  Future<int> clearStatsEventCallback() async {
    int reply = await _api.clearStatsEventCallback();
    return reply;
  }

  @override
  Future<int> joinChannel(String? token, String channelName, int uid,
      [NERtcJoinChannelOptions? channelOptions]) async {
    int reply;
    if (channelOptions != null) {
      JoinChannelOptions option = JoinChannelOptions(
          customInfo: channelOptions.customInfo,
          permissionKey: channelOptions.permissionKey);
      reply = await _api
          .joinChannel(JoinChannelRequest(channelName: channelName, uid: uid)
            ..token = token
            ..channelOptions = option);
    } else {
      reply = await _api.joinChannel(
          JoinChannelRequest(channelName: channelName, uid: uid)
            ..token = token);
    }
    return reply;
  }

  @override
  Future<int> switchChannel(String? token, String channelName,
      {NERtcJoinChannelOptions? channelOptions}) async {
    int reply;
    if (channelOptions != null) {
      JoinChannelOptions option = JoinChannelOptions(
          customInfo: channelOptions.customInfo,
          permissionKey: channelOptions.permissionKey);
      reply = await _api.switchChannel(SwitchChannelRequest()
        ..token = token
        ..channelName = channelName
        ..channelOptions = option);
    } else {
      reply = await _api.switchChannel(SwitchChannelRequest()
        ..token = token
        ..channelName = channelName);
    }
    return reply;
  }

  @override
  Future<int> leaveChannel() async {
    int reply = await _api.leaveChannel();
    return reply;
  }

  @override
  Future<int> updatePermissionKey(String key) async {
    int reply = await _api.updatePermissionKey(key);
    return reply;
  }

  @override
  Future<int> enableLocalAudio(bool enable) async {
    int reply = await _api.enableLocalAudio(enable);
    return reply;
  }

  @override
  Future<int> subscribeRemoteAudio(int uid, bool subscribe) async {
    int reply = await _api.subscribeRemoteAudio(SubscribeRemoteAudioRequest()
      ..subscribe = subscribe
      ..uid = uid);
    return reply;
  }

  @override
  Future<int> subscribeAllRemoteAudio(bool subscribe) async {
    int reply = await _api.subscribeAllRemoteAudio(subscribe);
    return reply;
  }

  @override
  Future<int> setAudioProfile(
      NERtcAudioProfile profile, NERtcAudioScenario scenario) async {
    int reply = await _api.setAudioProfile(SetAudioProfileRequest()
      ..profile = profile.index
      ..scenario = scenario.index);
    return reply;
  }

  @override
  Future<int> enableLocalVideo(bool enable, int streamType) async {
    int reply = await _api.enableLocalVideo(EnableLocalVideoRequest()
      ..enable = enable
      ..streamType = streamType);
    return reply;
  }

  @override
  Future<int> setLocalVideoConfig(
      NERtcVideoConfig videoConfig, int streamType) async {
    int reply = await _api.setLocalVideoConfig(SetLocalVideoConfigRequest()
      ..frontCamera = videoConfig.frontCamera
      ..videoCropMode = videoConfig.videoCropMode
      ..videoProfile = videoConfig.videoProfile
      ..frameRate = videoConfig.frameRate
      ..minFrameRate = videoConfig.minFrameRate
      ..bitrate = videoConfig.bitrate
      ..minBitrate = videoConfig.minBitrate
      ..degradationPrefer = videoConfig.degradationPrefer
      ..width = videoConfig.width
      ..height = videoConfig.height
      ..cameraType = videoConfig.cameraType
      ..mirrorMode = videoConfig.mirrorMode
      ..orientationMode = videoConfig.orientationMode
      ..streamType = streamType);
    return reply;
  }

  @override
  Future<int> setCameraCaptureConfig(
      NERtcCameraCaptureConfig captureConfig, int streamType) async {
    int reply =
        await _api.setCameraCaptureConfig(SetCameraCaptureConfigRequest()
          ..extraRotation = captureConfig.extraRotation
          ..captureWidth = max(0, captureConfig.captureWidth)
          ..captureHeight = max(0, captureConfig.captureHeight)
          ..streamType = streamType);
    return reply;
  }

  @override
  Future<int> startVideoPreview(int streamType) async {
    int reply = await _api.startVideoPreview(
        StartorStopVideoPreviewRequest()..streamType = streamType);
    return reply;
  }

  @override
  Future<int> stopVideoPreview(int streamType) async {
    int reply = await _api.stopVideoPreview(
        StartorStopVideoPreviewRequest()..streamType = streamType);
    return reply;
  }

  @override
  Future<int> startScreenCapture(NERtcScreenConfig config) async {
    int reply = await _api.startScreenCapture(StartScreenCaptureRequest()
      ..bitrate = config.bitrate
      ..contentPrefer = config.contentPrefer
      ..frameRate = config.frameRate
      ..minBitrate = config.minBitrate
      ..minFrameRate = config.minFrameRate
      ..videoProfile = config.videoProfile);
    return reply;
  }

  @override
  Future<int> enableLoopbackRecording(bool enable) async {
    int reply = await _api.enableLoopbackRecording(enable);
    return reply;
  }

  @override
  Future<int> stopScreenCapture() async {
    int reply = await _api.stopScreenCapture();
    return reply;
  }

  @override
  Future<int> subscribeRemoteSubStreamVideo(int uid, bool subscribe) async {
    int reply = await _api
        .subscribeRemoteSubStreamVideo(SubscribeRemoteSubStreamVideoRequest()
          ..uid = uid
          ..subscribe = subscribe);
    return reply;
  }

  @override
  Future<int> subscribeRemoteVideoStream(
      int uid, int streamType, bool subscribe) async {
    int reply = await _api
        .subscribeRemoteVideoStream(SubscribeRemoteVideoStreamRequest()
          ..uid = uid
          ..streamType = streamType
          ..subscribe = subscribe);
    return reply;
  }

  @override
  Future<int> enableAudioVolumeIndication(
      bool enable, int interval, bool vad) async {
    int reply = await _api
        .enableAudioVolumeIndication(EnableAudioVolumeIndicationRequest()
          ..enable = enable
          ..interval = interval
          ..vad = vad);
    return reply;
  }

  @override
  Future<int> muteLocalAudioStream(bool mute) async {
    int reply = await _api.muteLocalAudioStream(mute);
    return reply;
  }

  @override
  Future<int> muteLocalVideoStream(bool mute, int streamType) async {
    int reply = await _api.muteLocalVideoStream(mute, streamType);
    return reply;
  }

  @override
  Future<int> muteLocalSubStreamAudio(bool muted) async {
    int reply = await _api.muteLocalSubStreamAudio(muted);
    return reply;
  }

  @override
  Future<int> enableLocalSubStreamAudio(bool enable) async {
    int reply = await _api.enableLocalSubStreamAudio(enable);
    return reply;
  }

  @override
  Future<int> startAudioDump() async {
    int reply = await _api.startAudioDump();
    return reply;
  }

  @override
  Future<int> stopAudioDump() async {
    int reply = await _api.stopAudioDump();
    return reply;
  }

  @override
  Future<int> addLiveStreamTask(NERtcLiveStreamTaskInfo taskInfo) async {
    int serial = -1;
    List<Map<dynamic, dynamic>>? userTranscodingList =
        taskInfo.layout?.userTranscodingList?.map((e) => e.toMap()).toList();
    int reply = await _api.addLiveStreamTask(AddOrUpdateLiveStreamTaskRequest()
      ..serial = serial
      ..taskId = taskInfo.taskId
      ..url = taskInfo.url
      ..serverRecordEnabled = taskInfo.serverRecordEnabled
      ..liveMode = taskInfo.liveMode
      ..layoutWidth = taskInfo.layout?.width
      ..layoutHeight = taskInfo.layout?.height
      ..layoutBackgroundColor = taskInfo.layout?.backgroundColor?.value
      ..layoutImageUrl = taskInfo.layout?.backgroundImg?.url
      ..layoutImageX = taskInfo.layout?.backgroundImg?.x
      ..layoutImageY = taskInfo.layout?.backgroundImg?.y
      ..layoutImageWidth = taskInfo.layout?.backgroundImg?.width
      ..layoutImageHeight = taskInfo.layout?.backgroundImg?.height
      ..layoutUserTranscodingList = userTranscodingList);
    return reply;
  }

  @override
  Future<int> enableDualStreamMode(bool enable) async {
    int reply = await _api.enableDualStreamMode(enable);
    return reply;
  }

  @override
  Future<int> setChannelProfile(int channelProfile) async {
    int reply = await _api.setChannelProfile(channelProfile);
    return reply;
  }

  @override
  Future<int> updateLiveStreamTask(NERtcLiveStreamTaskInfo taskInfo) async {
    int serial = -1;
    List<Map<dynamic, dynamic>>? userTranscodingList =
        taskInfo.layout?.userTranscodingList?.map((e) => e.toMap()).toList();

    int reply =
        await _api.updateLiveStreamTask(AddOrUpdateLiveStreamTaskRequest()
          ..serial = serial
          ..taskId = taskInfo.taskId
          ..url = taskInfo.url
          ..serverRecordEnabled = taskInfo.serverRecordEnabled
          ..liveMode = taskInfo.liveMode
          ..layoutWidth = taskInfo.layout?.width
          ..layoutHeight = taskInfo.layout?.height
          ..layoutBackgroundColor = taskInfo.layout?.backgroundColor?.value
          ..layoutImageUrl = taskInfo.layout?.backgroundImg?.url
          ..layoutImageX = taskInfo.layout?.backgroundImg?.x
          ..layoutImageY = taskInfo.layout?.backgroundImg?.y
          ..layoutImageWidth = taskInfo.layout?.backgroundImg?.width
          ..layoutImageHeight = taskInfo.layout?.backgroundImg?.height
          ..layoutUserTranscodingList = userTranscodingList);
    return reply;
  }

  @override
  Future<int> removeLiveStreamTask(String taskId) async {
    int serial = -1;
    int reply = await _api.removeLiveStreamTask(DeleteLiveStreamTaskRequest()
      ..serial = serial
      ..taskId = taskId);
    return reply;
  }

  @override
  Future<int> adjustPlaybackSignalVolume(int volume) async {
    int reply = await _api.adjustPlaybackSignalVolume(volume);
    return reply;
  }

  @override
  Future<int> adjustRecordingSignalVolume(int volume) async {
    int reply = await _api.adjustRecordingSignalVolume(volume);
    return reply;
  }

  @override
  Future<int> getConnectionState() async {
    int reply = await _api.getConnectionState();
    return reply;
  }

  @override
  Future<int> sendSEIMsg(String seiMsg, int streamType) async {
    int reply = await _api.sendSEIMsg(SendSEIMsgRequest()
      ..seiMsg = seiMsg
      ..streamType = streamType);
    return reply;
  }

  @override
  Future<int> setClientRole(int role) async {
    int reply = await _api.setClientRole(role);
    return reply;
  }

  @override
  Future<int> uploadSdkInfo() async {
    int reply = await _api.uploadSdkInfo();
    return reply;
  }

  @override
  Future<int> setAudioEffectPreset(int preset) async {
    int reply = await _api.setAudioEffectPreset(preset);
    return reply;
  }

  @override
  Future<int> setLocalVoiceEqualization(int bandFrequency, int bandGain) async {
    int reply =
        await _api.setLocalVoiceEqualization(SetLocalVoiceEqualizationRequest()
          ..bandFrequency = bandFrequency
          ..bandGain = bandGain);
    return reply;
  }

  @override
  Future<int> setVoiceBeautifierPreset(int preset) async {
    int reply = await _api.setVoiceBeautifierPreset(preset);
    return reply;
  }

  @override
  Future<int> setLocalVoicePitch(double pitch) async {
    int reply = await _api.setLocalVoicePitch(pitch);
    return reply;
  }

  @override
  Future<int> setLocalVoiceReverbParam(NERtcReverbParam param) async {
    int reply =
        await _api.setLocalVoiceReverbParam(SetLocalVoiceReverbParamRequest()
          ..damping = param.damping
          ..decayTime = param.decayTime
          ..dryGain = param.dryGain
          ..preDelay = param.preDelay
          ..roomSize = param.roomSize
          ..wetGain = param.wetGain);
    return reply;
  }

  @override
  Future<int> startAudioRecording(
      String filePath, int sampleRate, int quality) async {
    int reply = await _api.startAudioRecording(StartAudioRecordingRequest()
      ..filePath = filePath
      ..sampleRate = sampleRate
      ..quality = quality);
    return reply;
  }

  Future<int> startAudioRecordingWithConfig(
      NERtcAudioRecordingConfiguration config) async {
    int reply = await _api
        .startAudioRecordingWithConfig(AudioRecordingConfigurationRequest()
          ..filePath = config.filePath
          ..sampleRate = config.sampleRate
          ..quality = config.quality
          ..cycleTime = config.cycleTime
          ..position = config.position);
    return reply;
  }

  @override
  Future<int> setLocalMediaPriority(int priority, bool isPreemptive) async {
    int reply = await _api.setLocalMediaPriority(SetLocalMediaPriorityRequest()
      ..isPreemptive = isPreemptive
      ..priority = priority);
    return reply;
  }

  @override
  Future<int> stopAudioRecording() async {
    int reply = await _api.stopAudioRecording();
    return reply;
  }

  @override
  Future<int> startChannelMediaRelay(
      NERtcChannelMediaRelayConfiguration config) async {
    StartOrUpdateChannelMediaRelayRequest request =
        StartOrUpdateChannelMediaRelayRequest();
    request.sourceMediaInfo = config.sourceMediaInfo?.toMap();
    request.destMediaInfo = {};
    config.destMediaInfo.forEach((key, value) {
      request.destMediaInfo![key] = value.toMap();
    });
    int reply = await _api.startChannelMediaRelay(request);
    return reply;
  }

  @override
  Future<int> stopChannelMediaRelay() async {
    int reply = await _api.stopChannelMediaRelay();
    return reply;
  }

  @override
  Future<int> updateChannelMediaRelay(
      NERtcChannelMediaRelayConfiguration config) async {
    StartOrUpdateChannelMediaRelayRequest request =
        StartOrUpdateChannelMediaRelayRequest();
    request.sourceMediaInfo = config.sourceMediaInfo?.toMap();
    request.destMediaInfo = {};
    config.destMediaInfo.forEach((key, value) {
      request.destMediaInfo![key] = value.toMap();
    });
    int reply = await _api.updateChannelMediaRelay(request);
    return reply;
  }

  @override
  Future<int> reportCustomEvent(String eventName, String? customIdentify,
      Map<String?, Object?>? param) async {
    int reply = await _api.reportCustomEvent(ReportCustomEventRequest()
      ..customIdentify = customIdentify
      ..eventName = eventName
      ..param = param);
    return reply;
  }

  @override
  Future<int> adjustUserPlaybackSignalVolume(int uid, int volume) async {
    int reply = await _api
        .adjustUserPlaybackSignalVolume(AdjustUserPlaybackSignalVolumeRequest()
          ..uid = uid
          ..volume = volume);
    return reply;
  }

  @override
  Future<int> enableEncryption(
      bool enable, NERtcEncryptionConfig config) async {
    int reply = await _api.enableEncryption(EnableEncryptionRequest()
      ..enable = enable
      ..key = config.key
      ..mode = config.mode.index);
    return reply;
  }

  @override
  Future<int> enableSuperResolution(bool enable) async {
    int reply = await _api.enableSuperResolution(enable);
    return reply;
  }

  @override
  Future<int> setAudioSessionOperationRestriction(
      NERtcAudioSessionOperationRestriction restriction) async {
    int reply =
        await _api.setAudioSessionOperationRestriction(restriction.index);
    return reply;
  }

  @override
  Future<int> setLocalPublishFallbackOption(int option) async {
    int reply = await _api.setLocalPublishFallbackOption(option);
    return reply;
  }

  @override
  Future<int> setRemoteSubscribeFallbackOption(int option) async {
    int reply = await _api.setRemoteSubscribeFallbackOption(option);
    return reply;
  }

  @override
  Future<int> getEffectDuration(int effectId) async {
    int reply = await _api.getEffectDuration(effectId);
    return reply;
  }

  @override
  Future<int> startLastmileProbeTest(LastmileProbeConfig config) async {
    int reply =
        await _api.startLastmileProbeTest(StartLastmileProbeTestRequest()
          ..probeUplink = config.probeUplink
          ..probeDownlink = config.probeDownlink
          ..expectedUplinkBitrate = config.expectedUplinkBitrate
          ..expectedDownlinkBitrate = config.expectedDownlinkBitrate);
    return reply;
  }

  @override
  Future<int> stopLastmileProbeTest() async {
    int reply = await _api.stopLastmileProbeTest();
    return reply;
  }

  @override
  Future<int> setVideoCorrectionConfig(
      NERtcVideoCorrectionConfiguration? config) async {
    int reply =
        await _api.setVideoCorrectionConfig(SetVideoCorrectionConfigRequest()
          ..topLeft = config?.topLeft
          ..topRight = config?.topRight
          ..bottomLeft = config?.bottomLeft
          ..bottomRight = config?.bottomRight
          ..canvasWidth = config?.canvasWidth
          ..canvasHeight = config?.canvasHeight
          ..enableMirror = config?.enableMirror);
    return reply;
  }

  @override
  Future<int> enableVirtualBackground(
      bool enabled, NERtcVirtualBackgroundSource? backgroundSource) async {
    int reply;
    if (backgroundSource == null) {
      reply = await _api.enableVirtualBackground(
          EnableVirtualBackgroundRequest()..enabled = enabled);
      return reply;
    }
    reply = await _api.enableVirtualBackground(EnableVirtualBackgroundRequest()
      ..enabled = enabled
      ..backgroundSourceType = backgroundSource.backgroundSourceType
      ..source = backgroundSource.source
      ..color = backgroundSource.color
      ..blur_degree = backgroundSource.blur_degree);
    return reply;
  }

  @override
  Future<int> setCloudProxy(int proxyType) async {
    int reply = await _api.setCloudProxy(proxyType);
    return reply;
  }

  @override
  Future<int> setRemoteHighPriorityAudioStream(
      bool enabled, int uid, int streamType) async {
    int reply = await _api.setRemoteHighPriorityAudioStream(
        SetRemoteHighPriorityAudioStreamRequest()
          ..enabled = enabled
          ..uid = uid
          ..streamType = streamType);
    return reply;
  }

  @override
  Future<int> startBeauty() async {
    int reply = await _api.startBeauty();
    return reply;
  }

  @override
  Future<void> stopBeauty() async {
    await _api.stopBeauty();
    return;
  }

  @override
  Future<int> enableBeauty(bool enabled) async {
    int reply = await _api.enableBeauty(enabled);
    return reply;
  }

  @override
  Future<int> setBeautyEffect(double level, int beautyType) async {
    int reply = await _api.setBeautyEffect(level, beautyType);
    return reply;
  }

  @override
  Future<int> addBeautyFilter(String path, String name) async {
    int reply = await _api.addBeautyFilter(path, name);
    return reply;
  }

  @override
  Future<void> removeBeautyFilter() async {
    await _api.removeBeautyFilter();
    return;
  }

  @override
  Future<int> setBeautyFilterLevel(double level) async {
    int reply = await _api.setBeautyFilterLevel(level);
    return reply;
  }

  @override
  Future<int> setLocalVideoWatermarkConfigs(
      int type, NERtcVideoWatermarkConfig? config) async {
    int reply;
    if (config == null) {
      reply = await _api.setLocalVideoWatermarkConfigs(
          SetLocalVideoWatermarkConfigsRequest(type: type));
    } else {
      VideoWatermarkConfig? _videoWatermarkConfig =
          convertToVideoWatermarkConfig(config);
      if (_videoWatermarkConfig == null) {
        return -1;
      }
      reply = await _api.setLocalVideoWatermarkConfigs(
          SetLocalVideoWatermarkConfigsRequest(type: type)
            ..config = _videoWatermarkConfig);
    }
    return reply;
  }

  VideoWatermarkConfig? convertToVideoWatermarkConfig(
      NERtcVideoWatermarkConfig nertcConfig) {
    if (nertcConfig.WatermarkType ==
        NERtcVideoWatermarkType.kNERtcVideoWatermarkTypeTimeStamp) {
      final timestampConfig = VideoWatermarkTimestampConfig(
        wmAlpha: nertcConfig.timestampWatermark?.wmAlpha,
        wmWidth: nertcConfig.timestampWatermark?.wmWidth,
        wmHeight: nertcConfig.timestampWatermark?.wmHeight,
        offsetX: nertcConfig.timestampWatermark?.offsetX,
        offsetY: nertcConfig.timestampWatermark?.offsetY,
        wmColor: nertcConfig.timestampWatermark?.wmColor,
        fontSize: nertcConfig.timestampWatermark?.fontSize,
        fontColor: nertcConfig.timestampWatermark?.fontColor,
        fontNameOrPath: nertcConfig.timestampWatermark?.fontNameOrPath,
      );
      return VideoWatermarkConfig(
          WatermarkType:
              NERtcVideoWatermarkType.kNERtcVideoWatermarkTypeTimeStamp,
          timestampWatermark: timestampConfig);
    } else if (nertcConfig.WatermarkType ==
        NERtcVideoWatermarkType.kNERtcVideoWatermarkTypeImage) {
      final imageConfig = VideoWatermarkImageConfig(
        wmAlpha: nertcConfig.imageWatermark?.wmAlpha,
        wmWidth: nertcConfig.imageWatermark?.wmWidth,
        wmHeight: nertcConfig.imageWatermark?.wmHeight,
        offsetX: nertcConfig.imageWatermark?.offsetX,
        offsetY: nertcConfig.imageWatermark?.offsetY,
        imagePaths:
            nertcConfig.imageWatermark?.imagePaths.map((e) => e ?? '').toList(),
        fps: nertcConfig.imageWatermark?.fps,
        loop: nertcConfig.imageWatermark?.loop,
      );
      return VideoWatermarkConfig(
          WatermarkType: NERtcVideoWatermarkType.kNERtcVideoWatermarkTypeImage,
          imageWatermark: imageConfig);
    } else if (nertcConfig.WatermarkType ==
        NERtcVideoWatermarkType.kNERtcVideoWatermarkTypeText) {
      final textConfig = VideoWatermarkTextConfig(
        wmAlpha: nertcConfig.textWatermark?.wmAlpha,
        wmWidth: nertcConfig.textWatermark?.wmWidth,
        wmHeight: nertcConfig.textWatermark?.wmHeight,
        offsetX: nertcConfig.textWatermark?.offsetX,
        offsetY: nertcConfig.textWatermark?.offsetY,
        wmColor: nertcConfig.textWatermark?.wmColor,
        fontSize: nertcConfig.textWatermark?.fontSize,
        fontColor: nertcConfig.textWatermark?.fontColor,
        fontNameOrPath: nertcConfig.textWatermark?.fontNameOrPath,
        content: nertcConfig.textWatermark?.content,
      );
      return VideoWatermarkConfig(
          WatermarkType: NERtcVideoWatermarkType.kNERtcVideoWatermarkTypeText,
          textWatermark: textConfig);
    }
    return null;
  }

  @override
  Future<int> startAudioDumpWithType(int dumpType) async {
    int reply = await _api.startAudioDumpWithType(dumpType);
    return reply;
  }

  @override
  Future<int> adjustLoopBackRecordingSignalVolume(int volume) async {
    int reply = await _api.adjustLoopBackRecordingSignalVolume(volume);
    return reply;
  }

  @override
  Future<int> getNtpTimeOffset() async {
    int reply = await _api.getNtpTimeOffset();
    return reply;
  }

  @override
  Future<int> setStreamAlignmentProperty(bool enable) async {
    int reply = await _api.setStreamAlignmentProperty(enable);
    return reply;
  }

  @override
  Future<int> enableMediaPub(int mediaType, bool enable) async {
    int reply = await _api.enableMediaPub(mediaType, enable);
    return reply;
  }

  @override
  Future<int> setAudioSubscribeOnlyBy(List<int>? uidArray) async {
    int reply = await _api.setAudioSubscribeOnlyBy(
        SetAudioSubscribeOnlyByRequest()..uidArray = uidArray);
    return reply;
  }

  @override
  Future<int> subscribeRemoteSubStreamAudio(int uid, bool subscribe) async {
    int reply = await _api
        .subscribeRemoteSubStreamAudio(SubscribeRemoteSubStreamAudioRequest()
          ..uid = uid
          ..subscribe = subscribe);
    return reply;
  }

  @override
  Future<int> takeLocalSnapshot(int streamType, String path) async {
    int reply = await _api.takeLocalSnapshot(streamType, path);
    return reply;
  }

  @override
  Future<int> takeRemoteSnapshot(int uid, int streamType, String path) async {
    int reply = await _api.takeRemoteSnapshot(uid, streamType, path);
    return reply;
  }

  @override
  Future<int> enableVideoCorrection(bool enable) async {
    int reply = await _api.enableVideoCorrection(enable);
    return reply;
  }

  @override
  Future<int> setExternalVideoSource(int streamType, bool enable) async {
    int reply = await _api.setExternalVideoSource(streamType, enable);
    return reply;
  }

  @override
  Future<int> pushExternalVideoFrame(
      int streamType, NERtcVideoFrame frame) async {
    int reply = await _api.pushExternalVideoFrame(
        streamType,
        VideoFrame()
          ..data = frame.data
          ..width = frame.width
          ..height = frame.height
          ..format = frame.format
          ..rotation = frame.rotation
          ..timeStamp = frame.timeStamp
          ..strideY = frame.strideY
          ..strideU = frame.strideU
          ..strideV = frame.strideV
          ..textureId = frame.textureId
          ..transformMatrix = frame.transformMatrix);
    return Future.value(reply);
  }
}

class MethodChannelRtcChannelEventSink extends NERtcChannelEventSink {
  final _listeners = <NERtcChannelEventSink>[];

  void addRtcChannelListener(NERtcChannelEventSink sink) {
    _listeners.add(sink);
  }

  void removeRtcChannelListener(NERtcChannelEventSink sink) {
    _listeners.remove(sink);
  }

  @override
  void onJoinChannel(int result, int channelId, int elapsed, int uid) {
    for (var element in _listeners) {
      element.onJoinChannel(result, channelId, elapsed, uid);
    }
  }

  @override
  void onLeaveChannel(int result) {
    for (var element in _listeners) {
      element.onLeaveChannel(result);
    }
  }

  @override
  void onUserJoined(UserJoinedEvent event) {
    for (var element in _listeners) {
      element.onUserJoined(event);
    }
  }

  @override
  void onUserLeave(UserLeaveEvent event) {
    for (var element in _listeners) {
      element.onUserLeave(event);
    }
  }

  @override
  void onDisconnect(int reason) {
    for (var element in _listeners) {
      element.onDisconnect(reason);
    }
  }

  @override
  void onUserAudioStart(int uid) {
    for (var element in _listeners) {
      element.onUserAudioStart(uid);
    }
  }

  @override
  void onUserAudioStop(int uid) {
    for (var element in _listeners) {
      element.onUserAudioStop(uid);
    }
  }

  @override
  void onUserSubStreamAudioStart(int uid) {
    for (var element in _listeners) {
      element.onUserSubStreamAudioStart(uid);
    }
  }

  @override
  void onUserSubStreamAudioStop(int uid) {
    for (var element in _listeners) {
      element.onUserSubStreamAudioStop(uid);
    }
  }

  @override
  void onUserVideoStart(int uid, int maxProfile) {
    for (var element in _listeners) {
      element.onUserVideoStart(uid, maxProfile);
    }
  }

  @override
  void onUserVideoStop(int uid) {
    for (var element in _listeners) {
      element.onUserVideoStop(uid);
    }
  }

  @override
  void onUserAudioMute(int uid, bool muted) {
    for (var element in _listeners) {
      element.onUserAudioMute(uid, muted);
    }
  }

  @override
  void onUserVideoMute(UserVideoMuteEvent event) {
    for (var element in _listeners) {
      element.onUserVideoMute(event);
    }
  }

  @override
  void onUserSubStreamAudioMute(int uid, bool muted) {
    for (var element in _listeners) {
      element.onUserSubStreamAudioMute(uid, muted);
    }
  }

  @override
  void onFirstAudioDataReceived(int uid) {
    for (var element in _listeners) {
      element.onFirstAudioDataReceived(uid);
    }
  }

  @override
  void onFirstAudioFrameDecoded(int uid) {
    for (var element in _listeners) {
      element.onFirstAudioFrameDecoded(uid);
    }
  }

  @override
  void onFirstVideoDataReceived(FirstVideoDataReceivedEvent event) {
    for (var element in _listeners) {
      element.onFirstVideoDataReceived(event);
    }
  }

  @override
  void onFirstVideoFrameDecoded(FirstVideoFrameDecodedEvent event) {
    for (var element in _listeners) {
      element.onFirstVideoFrameDecoded(event);
    }
  }

  @override
  void onVirtualBackgroundSourceEnabled(
      VirtualBackgroundSourceEnabledEvent event) {
    for (var element in _listeners) {
      element.onVirtualBackgroundSourceEnabled(event);
    }
  }

  @override
  void onConnectionTypeChanged(int newConnectionType) {
    for (var element in _listeners) {
      element.onConnectionTypeChanged(newConnectionType);
    }
  }

  @override
  void onReconnectingStart() {
    for (var element in _listeners) {
      element.onReconnectingStart();
    }
  }

  @override
  void onConnectionStateChanged(int state, int reason) {
    for (var element in _listeners) {
      element.onConnectionStateChanged(state, reason);
    }
  }

  @override
  void onReJoinChannel(int result, int channelId) {
    for (var element in _listeners) {
      element.onReJoinChannel(result, channelId);
    }
  }

  @override
  void onLocalAudioVolumeIndication(int volume, bool vadFlag) {
    for (var element in _listeners) {
      element.onLocalAudioVolumeIndication(volume, vadFlag);
    }
  }

  @override
  void onRemoteAudioVolumeIndication(RemoteAudioVolumeIndicationEvent event) {
    for (var element in _listeners) {
      element.onRemoteAudioVolumeIndication(event);
    }
  }

  @override
  void onClientRoleChange(int oldRole, int newRole) {
    for (var element in _listeners) {
      element.onClientRoleChange(oldRole, newRole);
    }
  }

  @override
  void onError(int code) {
    for (var element in _listeners) {
      element.onError(code);
    }
  }

  @override
  void onLiveStreamState(String taskId, String pushUrl, int liveState) {
    for (var element in _listeners) {
      element.onLiveStreamState(taskId, pushUrl, liveState);
    }
  }

  @override
  void onWarning(int code) {
    for (var element in _listeners) {
      element.onWarning(code);
    }
  }

  @override
  void onAudioHasHowling() {
    for (var element in _listeners) {
      element.onAudioHasHowling();
    }
  }

  @override
  void onUserSubStreamVideoStart(int uid, int maxProfile) {
    for (var element in _listeners) {
      element.onUserSubStreamVideoStart(uid, maxProfile);
    }
  }

  @override
  void onUserSubStreamVideoStop(int uid) {
    for (var element in _listeners) {
      element.onUserSubStreamVideoStop(uid);
    }
  }

  @override
  void onAudioRecording(int code, String filePath) {
    for (var element in _listeners) {
      element.onAudioRecording(code, filePath);
    }
  }

  @override
  void onMediaRelayStatesChange(int state, String channelName) {
    for (var element in _listeners) {
      element.onMediaRelayStatesChange(state, channelName);
    }
  }

  @override
  void onRecvSEIMsg(int userID, String seiMsg) {
    for (var element in _listeners) {
      element.onRecvSEIMsg(userID, seiMsg);
    }
  }

  @override
  void onLocalPublishFallbackToAudioOnly(bool isFallback, int streamType) {
    for (var element in _listeners) {
      element.onLocalPublishFallbackToAudioOnly(isFallback, streamType);
    }
  }

  @override
  void onMediaRelayReceiveEvent(int event, int code, String channelName) {
    for (var element in _listeners) {
      element.onMediaRelayReceiveEvent(event, code, channelName);
    }
  }

  @override
  void onRemoteSubscribeFallbackToAudioOnly(
      int uid, bool isFallback, int streamType) {
    for (var element in _listeners) {
      element.onRemoteSubscribeFallbackToAudioOnly(uid, isFallback, streamType);
    }
  }

  @override
  void onLocalVideoWatermarkState(int videoStreamType, int state) {
    for (var element in _listeners) {
      element.onLocalVideoWatermarkState(videoStreamType, state);
    }
  }

  @override
  void onLastmileProbeResult(NERtcLastmileProbeResult result) {
    for (var element in _listeners) {
      element.onLastmileProbeResult(result);
    }
  }

  @override
  void onLastmileQuality(int quality) {
    for (var element in _listeners) {
      element.onLastmileQuality(quality);
    }
  }

  @override
  void onMediaRightChange(
      bool isAudioBannedByServer, bool isVideoBannedByServer) {
    for (var element in _listeners) {
      element.onMediaRightChange(isAudioBannedByServer, isVideoBannedByServer);
    }
  }

  @override
  void onTakeSnapshotResult(int code, String path) {
    for (var element in _listeners) {
      element.onTakeSnapshotResult(code, path);
    }
  }
}
