// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'pigeon.dart';

/// 音视频通话相关的参数设置
class NERtcOptions {
  const NERtcOptions(
      {this.logDir,
      this.logLevel,
      this.serverAddresses,
      this.audioAutoSubscribe,
      this.videoAutoSubscribe,
      this.disableFirstJoinUserCreateChannel,
      this.audioDisableOverrideSpeakerOnReceiver,
      this.audioDisableSWAECOnHeadset,
      this.audioAINSEnabled,
      this.serverRecordAudio,
      this.serverRecordVideo,
      this.serverRecordMode,
      this.serverRecordSpeaker,
      this.publishSelfStream,
      this.videoEncodeMode,
      this.videoDecodeMode,
      this.videoCaptureObserverEnabled,
      this.videoSendMode,
      this.videoH265Enabled,
      this.mode1v1Enabled});

  /// 日志路径
  final String? logDir;

  /// 日志等级
  final int? logLevel;

  /// 自定义服务器地址，用于私有化部署
  final NERtcServerAddresses? serverAddresses;

  /// 是否自动订阅音频（默认订阅）
  final bool? audioAutoSubscribe;

  /// 是否自动订阅视频（默认不订阅）
  final bool? videoAutoSubscribe;

  /// 加入房间时禁止创建房间（暂时不支持该功能）
  final bool? disableFirstJoinUserCreateChannel;

  /// 系统切换听筒事件时，禁用切换到扬声器.
  /// 仅在iOS平台有效.
  /// 默认值 false, 如设置 true 则禁止SDK在系统切换到听筒时做切换扬声器操作，需要用户自己处理切换听筒事件
  final bool? audioDisableOverrideSpeakerOnReceiver;

  /// 设置耳机时不使用软件回声消除功能
  /// 仅在iOS平台有效.
  /// 默认值 false,如设置true 则SDK在耳机模式下不使用软件回声消除功能，会对某些机型下 耳机的音质效果有影响
  final bool? audioDisableSWAECOnHeadset;

  /// 设置是否开启AI降噪，开启AI降噪，在嘈杂环境下，让对方更清晰听到您的声音
  final bool? audioAINSEnabled;

  /// 是否开启服务器录制语音
  final bool? serverRecordAudio;

  /// 是否开启服务器录制视频
  final bool? serverRecordVideo;

  /// 服务器录制模式
  final NERtcServerRecordMode? serverRecordMode;

  /// 是否服务器录制主讲人
  final bool? serverRecordSpeaker;

  /// 是否允许在房间推流时推送自身的视频流 （该参数已废弃）
  final bool? publishSelfStream;

  /// 是否允许视频帧回调.
  /// 仅在iOS平台有效.
  final bool? videoCaptureObserverEnabled;

  /// 视频编码模式
  final NERtcMediaCodecMode? videoEncodeMode;

  /// 视频解码模式.
  final NERtcMediaCodecMode? videoDecodeMode;

  /// 视频发布模式
  final NERtcVideoSendMode? videoSendMode;

  ///是否开启H265
  final bool? videoH265Enabled;

  /// 是否开启频道1V1模式，joinChannel 前设置生效
  final bool? mode1v1Enabled;

  @override
  String toString() {
    return 'NERtcOptions{logDir:$logDir,logLevel:$logLevel,'
        'serverAddresses:${serverAddresses.toString()},audioAutoSubscribe:$audioAutoSubscribe,'
        'videoAutoSubscribe:$videoAutoSubscribe,disableFirstJoinUserCreateChannel:$disableFirstJoinUserCreateChannel,'
        'audioDisableOverrideSpeakerOnReceiver:$audioDisableOverrideSpeakerOnReceiver,audioDisableSWAECOnHeadset:$audioDisableSWAECOnHeadset,'
        'audioAINSEnabled:$audioAINSEnabled,serverRecordAudio:$serverRecordAudio,serverRecordVideo:$serverRecordVideo,'
        'serverRecordMode:$serverRecordMode,serverRecordSpeaker:$serverRecordSpeaker,publishSelfStream:$publishSelfStream,'
        'videoEncodeMode:$videoEncodeMode,videoDecodeMode:$videoDecodeMode,'
        'videoCaptureObserverEnabled:$videoCaptureObserverEnabled,videoSendMode:$videoSendMode,'
        'videoH265Enabled:$videoH265Enabled,mode1v1Enabled:$mode1v1Enabled}';
  }
}

///joinChannel 时的一些可选信息
class NERtcJoinChannelOptions {
  ///自定义信息，最长支持127个字符, 若超过127个字符，该参数无效。
  final String? customInfo;

  ///权限密钥。能控制通话时长及媒体权限能力。(高级鉴权功能，暂时不支持)
  final String? permissionKey;

  NERtcJoinChannelOptions(this.customInfo, this.permissionKey);
  @override
  String toString() {
    return 'NERtcJoinChannelOptions{customInfo:$customInfo,permissionKey:$permissionKey}';
  }
}

/// Camera类型
/// 仅 Android 平台支持
class NERtcCameraType {
  /// 使用 android.hardware.camera 进行视频采集
  /// 默认情况下使用 [camera1]
  static const int camera1 = 1;

  /// 使用 android.hardware.camera2 进行视频采集
  static const int camera2 = 2;
}

///指定的摄像头位置
class NERtcCameraPosition {
  ///前置摄像头。
  static const int kNERtcCameraPositionBack = 0;

  ///后置摄像头。
  static const int kNERtcCameraPositionFront = 1;
}

///屏幕共享编码策略倾向
class NERtcSubStreamContentPrefer {
  /// （默认）内容类型为动画。当共享的内容是视频、电影或游戏时，推荐用户选择该内容类型
  static const int motion = 0;

  /// 内容类型为细节。当共享的内容是图片或文字时，推荐选择该内容类型（该模式下帧率最高10帧）
  static const int details = 1;
}

/// 摄像头采集配置
///
/// 如果您需要自定义本地采集的视频尺寸，请先将 preference 设为 [NERtcCameraCapturePreference.kManual]，再通过 captureWidth 和 captureHeight 设置采集的视频宽度。手动模式下，如果指定的采集宽高小于编码宽高，编码参数会被下调对齐到采集的尺寸范围内。
class NERtcCameraCaptureConfig {
  /// 设置摄像头的额外旋转信息。
  /// [NERtcCaptureExtraRotation]
  final NERtcCaptureExtraRotation extraRotation;

  /// 本地采集的视频宽度，单位为 px。
  /// 视频编码分辨率以宽 x 高表示，用于设置视频编码分辨率，以衡量编码质量。
  final int captureWidth;

  /// 本地采集的视频高度，单位为 px。
  /// 视频编码分辨率以宽 x 高表示，用于设置视频编码分辨率，以衡量编码质量。
  final int captureHeight;

  const NERtcCameraCaptureConfig(
      this.extraRotation, this.captureWidth, this.captureHeight);

  NERtcCameraCaptureConfig.manual(int captureWidth, int captureHeight)
      : this(NERtcCaptureExtraRotation.kNERtcCaptureExtraRotationDefault,
            captureWidth, captureHeight);
  @override
  String toString() {
    return 'NERtcCameraCaptureConfig{captureWidth:$captureWidth,captureHeight:$captureHeight,extraRotation:$extraRotation}';
  }
}

/// 视频设置参数
class NERtcVideoConfig {
  /// 视频编码的分辨率，用于衡量编码质量视频档位。 详细信息请参考 [NERtcVideoProfile]。
  int videoProfile = NERtcVideoProfile.standard;

  /// 视频画面裁剪模式，默认为 [NERtcVideoCropMode.cropDefault]。自定义视频输入不支持设置裁剪模式。
  int videoCropMode = NERtcVideoCropMode.cropDefault;

  /// 摄像头位置，默认前置摄像头
  bool frontCamera = true;

  /// 视频编码的帧率。详细信息请参考 [NERtcVideoFrameRate]。
  int frameRate = NERtcVideoFrameRate.fpsDefault;

  /// 视频编码的最小帧率。默认为 0，表示使用默认最小帧率。
  int minFrameRate = 0;

  /// 视频编码的码率，单位为 Kbps。您可以根据场景需要，手动设置想要的码率。
  ///
  /// - 若设置的视频码率超出合理范围，SDK 会自动按照合理区间处理码率。
  /// - 若设置为 0，SDK将会自行计算处理。
  int bitrate = 0;

  /// 视频编码的最小码率，单位为 Kbps。您可以根据场景需要，手动设置想要的最小码率，若设置为0，SDK 将会自行计算处理。
  int minBitrate = 0;

  /// 带宽受限时的视频编码降级偏好。详细信息请参考 [NERtcDegradationPreference]。`
  int degradationPrefer = NERtcDegradationPreference.degradationDefault;

  /// 视频编码分辨率，衡量编码质量，以宽x高表示。与maxProfile属性二选一。
  /// width表示视频帧在横轴上的像素，即自定义宽。
  ///
  /// - 设置为负数时表示采用 maxProfile档位。
  /// - 如果需要自定义分辨率场景，则设置此属性，maxProfile属性失效。
  ///
  /// 自定义视频输入 width 和 height 无效，会自动根据 maxProfile缩放。
  int width = 0;

  /// 视频编码分辨率，衡量编码质量，以宽x高表示。与maxProfile属性二选一。
  /// height表示视频帧在纵轴上的像素，即自定义高。
  ///
  /// - 设置为负数时表示采用 maxProfile档位。
  /// - 如果需要自定义分辨率场景，则设置此属性，maxProfile属性失效。
  ///
  /// 自定义视频输入width和height无效，会自动根据 maxProfile缩放。
  int height = 0;

  /// Camera 类型，仅 Android 平台支持
  int cameraType = NERtcCameraType.camera1;

  /// 设置本地视频编码的镜像模式，即本地发送视频的镜像模式，只影响远端用户看到的视频画面。 详细信息请参考 [NERtcVideoMirrorMode]。
  int mirrorMode = NERtcVideoMirrorMode.auto;

  /// 设置本地视频编码的方向模式，即本地发送视频的方向模式，只影响远端用户看到的视频画面。 详细信息请参考 {@link NERtcVideoOutputOrientationMode}。
  int orientationMode = NERtcVideoOutputOrientationMode.adaptative;

  @override
  String toString() {
    return 'NERtcVideoConfig{videoProfile: $videoProfile, '
        'videoCropMode: $videoCropMode, frontCamera: $frontCamera, '
        'frameRate: $frameRate, minFrameRate: $minFrameRate, '
        'bitrate: $bitrate, minBitrate: $minBitrate, '
        'degradationPrefer: $degradationPrefer, width: $width, height: $height, '
        'cameraType: $cameraType, mirrorMode: $mirrorMode, '
        'orientationMode: $orientationMode}';
  }
}

/// 屏幕录制编码参数
class NERtcScreenConfig {
  /// 屏幕共享编码策略倾向
  int contentPrefer = NERtcSubStreamContentPrefer.motion;

  /// 视频档位，默认高清模式
  int videoProfile = NERtcVideoProfile.standard;

  /// 视频编码帧率
  int frameRate = NERtcVideoFrameRate.fps_7;

  /// 最小帧率。0：表示使用默认帧率
  int minFrameRate = 0;

  /// 视频编码码率，单位为Kbps。 0：表示使用默认码率，手动设置请参考码表
  int bitrate = 0;

  /// 视频编码最小码率，单位为Kbps。直播场景下使用，0：表示使用默认
  int minBitrate = 0;

  @override
  String toString() {
    return 'NERtcScreenConfig{contentPrefer: $contentPrefer, '
        'videoProfile: $videoProfile, frameRate: $frameRate, '
        'minFrameRate: $minFrameRate, bitrate: $bitrate, minBitrate: $minBitrate}';
  }
}

/// 摄像头采集偏好。
class NERtcCameraCapturePreference {
  /// （默认）优先保证设备性能。SDK 根据设备性能，参考用户在 setLocalVideoConfig 中设置编码器的分辨率和帧率，选择最接近的摄像头输出参数。在这种情况下，预览质量接近于编码器的输出质量。
  static const int kDefault = 0;

  /// 优先保证视频预览质量。SDK 自动设置画质较高的摄像头输出参数，提高预览画面质量。此时会消耗更多的 CPU 及内存做视频前处理。
  static const int kQuality = 1;

  /// 采用用户自定义设置的摄像头输出参数。此时用户可以通过 NERtcCameraCaptureConfig 中的 captureWidth 和 captureHeight 设置本地摄像头采集的视频宽高。
  static const int kManual = 2;
}

///视频清晰度
class NERtcVideoProfile {
  ///LD (160 × 120/90 @ 15fps)
  static const int lowest = 0;

  ///标清,（320x180/240 @15fps）
  static const int low = 1;

  ///高清, (640x360/480 @30fps)
  static const int standard = 2;

  ///超清, (1280x720 @30 fps)
  static const int hd720p = 3;

  ///1080P, (1920x1080 @30fps)
  static const int hd1080p = 4;

  static const int max = hd1080p;
}

/// 视频镜像模式
class NERtcVideoMirrorMode {
  /// （默认）由 SDK 决定镜像模式
  static const int auto = 0;

  /// 启用镜像模式
  static const int enabled = 1;

  /// 关闭镜像模式
  static const int disabled = 2;
}

/// 视频旋转方向模式
class NERtcVideoOutputOrientationMode {
  ///（默认）该模式下 SDK 输出的视频方向与采集到的视频方向一致。接收端会根据收到的视频旋转信息对视频进行旋转。
  ///
  /// 该模式适用于接收端可以调整视频方向的场景。
  /// - 如果采集的视频是横屏模式，则输出的视频也是横屏模式。
  /// - 如果采集的视频是竖屏模式，则输出的视频也是竖屏模式。
  static const int adaptative = 0;

  /// 该模式下 SDK 固定输出横屏模式的视频。如果采集到的视频是竖屏模式，则视频编码器会对其进行裁剪。
  ///
  /// 该模式适用于接收端无法调整视频方向的场景，例如旁路推流。
  static const int fixedLandscape = 1;

  /// 该模式下 SDK 固定输出竖屏模式的视频，如果采集到的视频是横屏模式，则视频编码器会对其进行裁剪。
  ///
  /// 该模式适用于接收端无法调整视频方向的场景，例如旁路推流。
  static const int fixedPortrait = 2;
}

///屏幕共享清晰度
class NERtcScreenProfile {
  ///高清, (640x360/480 @5fps)
  static const int hd480p = 0;

  ///超清, (1280x720 @5 fps)
  static const int hd720p = 1;

  ///1080P, (1920x1080 @5fps)
  static const int hd1080p = 2;
}

///视频画布缩放方式
enum NERtcVideoViewFitType {
  /// As large as possible while still containing the source entirely within the
  /// target box.
  contain,

  /// As small as possible while still covering the entire target box.
  cover,
}

///视频裁剪模式
class NERtcVideoCropMode {
  ///相机原始比例
  static const int cropDefault = 0;

  ///16:9 裁剪
  static const int crop_16x9 = 1;

  ///4:3 裁剪
  static const int crop_4x3 = 2;

  ///1:1 裁剪
  static const int crop_1x1 = 3;
}

/// 视频流类型
class NERtcVideoStreamType {
  /// 主流
  static const int main = 0;

  /// 辅流
  static const int sub = 1;
}

///网络类型定义
class NERtcConnectionType {
  ///Unknown data connection.
  static const int unknown = 0;

  ///The Ethernet data connection.
  static const int ethernet = 1;

  ///The WIFI data connection.
  static const int wifi = 2;

  ///The Mobile(4G) data connection.
  static const int cellular4g = 3;

  ///The Mobile(3G) data connection.
  static const int cellular3g = 4;

  ///The Mobile(2G) data connection.
  static const int cellular2g = 5;

  /// The Unknown cellular data connection.
  static const int cellularUnknown = 6;

  ///The Bluetooth data connection.
  static const int bluetooth = 7;

  ///The VPN data connection.
  static const int vpn = 8;

  ///The Mobile(5G) data connection.
  static const int cellular5g = 9;

  ///The absence of a connection type.
  static const int none = 10;
}

///语音设备类型
class NERtcAudioDevice {
  /// 扬声器
  static const int speakerPhone = 0;

  /// 有线耳机
  static const int wiredHeadset = 1;

  /// 听筒
  static const int earpiece = 2;

  /// 蓝牙耳机
  static const int bluetoothHeadset = 3;
}

///音频设备状态
class NERtcAudioDeviceState {
  /// 初始化, Only iOS
  static const int initialized = 0;

  /// 打开成功
  static const int opened = 1;

  /// 已关闭
  static const int closed = 2;

  /// 初始化失败
  static const int initializationError = 3;

  /// 开启失败
  static const int startError = 4;

  /// 未知错误
  static const int unknownError = 5;

  /// 反初始化
  static const int unInitialized = 6;
}

///视频设备状态
class NERtcVideoDeviceState {
  /// 初始化, Only iOS
  static const int initialized = 0;

  ///打开成功
  static const int opened = 1;

  ///已关闭
  static const int closed = 2;

  ///相机断开，可能被其他应用抢占
  static const int disconnected = 3;

  ///相机冻结
  static const int freezed = 4;

  ///未知错误
  static const int unknownError = 5;

  /// 反初始化
  static const int unInitialized = 6;
}

///音频设备类型
class NERtcAudioDeviceType {
  /// 音频播放设备
  static const int playout = 2;

  /// 音频采集设备
  static const int record = 1;
}

class NERtcVideoDeviceType {
  ///未知视频设备
  static const int unknown = 0;

  ///视频采集设备
  static const int capture = 1;
}

///音频dump类型
class NERtcAudioDumpType {
  ///仅输出.dump文件（默认）
  static const int kNERtcAudioDumpTypePCM = 0;

  ///输出.dump和.wav文件
  static const int kNERtcAudioDumpTypeAll = 1;

  ///仅输出.wav文件
  static const int kNERtcAudioDumpTypeWAV = 2;
}

///虚拟背景未成功启用的原因或确认成功的信息
class NERtcVirtualBackgroundSourceStateReason {
  ///虚拟背景开启成功
  static const int kNERtcVirtualBackgroundSourceStateReasonSuccess = 0;

  ///自定义背景图片不存在。 请检查 VirtualBackgroundSource 中 source 的值
  static const int kNERtcVirtualBackgroundSourceStateReasonImageNotExist = 1;

  ///自定义背景图片的图片格式无效。 请检查 VirtualBackgroundSource 中 source 的值
  static const int
      kNERtcVirtualBackgroundSourceStateReasonImageFormatNotSupported = 2;

  ///自定义背景图片的颜色格式无效。 请检查 VirtualBackgroundSource 中 color 的值
  static const int
      kNERtcVirtualBackgroundSourceStateReasonColorFormatNotSupported = 3;

  ///该设备不支持使用虚拟背景
  static const int kNERtcVirtualBackgroundSourceStateReasonDeviceNotSupported =
      4;
}

///SEI发送的流通道类型
enum NERtcStreamChannelType {
  ///主流通道
  kNERtcStreamChannelTypeMainStream,

  ///辅流通道
  kNERtcStreamChannelTypeSubStream,
}

///音频应用场景
enum NERtcAudioScenario {
  ///默认设置为Speech
  scenarioDefault,

  ///语音场景. NERtcAudioProfile 推荐使用 profileMiddleQuality 及以下
  scenarioSpeech,

  ///音乐场景。NERtcAudioProfile 推荐使用 profileMiddleQualityStereo 及以上
  scenarioMusic,

  ///音乐场景。NERtcAudioProfile 推荐使用 profileHighQuality 及以上
  scenarioChatRoom,
}

///音频属性。设置采样率，码率，编码模式和声道数
enum NERtcAudioProfile {
  ///默认设置。Speech场景下为profileStandard，Music场景下为profileHighQuality
  profileDefault,

  ///标准音质模式。采样率为 16 kHz、语音编码、单声道、编码码率最大值为 20 Kbps
  profileStandard,

  ///标准扩展模式。采样率为 16 kHz、语音编码、单声道、编码码率最大值为 32 Kbps
  profileStandardExtend,

  ///中等音质模式。采样率为 48 kHz、音乐编码、单声道、编码码率最大值为 64 Kbps
  profileMiddleQuality,

  ///中等音质模式（立体音）。采样率为 48 kHz、音乐编码、双声道、编码码率最大值为 80 Kbps
  profileMiddleQualityStereo,

  ///高音质模式。采样率为 48 kHz、音乐编码、单声道、编码码率最大值为 96 Kbps
  profileHighQuality,

  ///高音质模式（立体音）。采样率为 48 kHz、音乐编码、双声道、编码码率最大值为 128 Kbps
  profileHighQualityStereo
}

/// 编解码模式，主要用来区分软件编解码和硬件编解码
enum NERtcMediaCodecMode {
  /// 优先使用硬件编解码器
  hardware,

  /// 优先使用软件编解码器
  software
}

/// 服务器录制模式
enum NERtcServerRecordMode {
  /// 混合与单人一起录制
  mixAndSingle,

  /// 混合录制
  mix,

  /// 单人录制
  single
}

/// 伴音任务状态更新
enum NERtcAudioMixingTaskState {
  ///伴音任务结束
  finished,

  /// 伴音任务出错
  error
}

///错误码定义
class NERtcErrorCode {
  /// 成功
  static const int ok = 0;

  ///参数错误
  static const int illegalArgument = -400;

  ///状态错误
  static const int illegalStatus = -500;

  ///未初始化
  static const int uninitialized = -501;

  ///操作不支持
  static const int invalidOperation = -700;

  ///请求超时
  static const int reserveTimeout = 408;

  ///参数错误
  static const int reserveInvalidParameter = 414;

  ///只支持两个用户, 有第三个人试图使用相同的频道名分配频道
  static const int reserveMoreThanTwoUser = 600;

  ///没有权限，包括没有开通音视频功能、没有开通非安全但是请求了非安全等
  static const int reserveNoPermission = 403;

  ///分配频道服务器出错
  static const int reserveServerFail = 500;

  ///内部错误
  static const int errorFatal = 30001;

  ///内存溢出
  static const int outOfMemory = 30002;

  ///参数错误
  static const int invalidParam = 30003;

  ///不支持
  static const int notSupported = 30004;

  ///状态错误
  static const int invalidState = 30005;

  ///缺乏资源
  static const int lackOfResource = 30006;

  ///序号非法
  static const int invalidIndex = 30007;

  ///设备未找到
  static const int deviceNotFound = 30008;

  ///设备ID非法
  static const int invalidDeviceSourceId = 30009;

  ///视频能力非法
  static const int invalidVideoProfile = 30010;

  ///创建设备失败
  static const int createDeviceSourceFail = 30011;

  ///画布非法
  static const int invalidRender = 30012;

  ///预览已打开
  static const int devicePreviewAlreadyStarted = 30013;

  ///挂起
  static const int transmitPending = 30014;

  ///连接失败
  static const int connectFail = 30015;

  ///创建dump 失败
  static const int createDumpFileFail = 30016;

  ///开始dump失败
  static const int startDumpFail = 30017;

  ///房间已加入
  static const int roomAlreadyJoined = 30100;

  ///房间未加入
  static const int roomNotJoined = 30101;

  ///重复离开房间
  static const int roomRepeatedlyLeave = 30102;

  ///请求加入房间失败
  static const int requestJoinRoomFail = 30103;

  ///会话未找到
  static const int sessionNotFound = 30104;

  ///用户未找到
  static const int userNotFound = 30105;

  ///非法用户
  static const int invalidUserId = 30106;

  ///媒体会话未建立
  static const int mediaNotStarted = 30107;

  ///媒体源未找到
  static const int sourceNotFound = 30108;

  ///切换房间状态无效
  static const int switchChannelInvalidState = 30109;

  /// 原因通常为重复调用 startChannelMediaRelay。成功调用startChannelMediaRelay后，必须先调用 stopChannelMediaRelay 方法退出当前的转发状态，才能再次调用该方法
  static const int channelMediaRelayInvalidState = 30110;

  /// 媒体流转发权限不足
  ///
  /// 原因通常包括：
  /// - 源房间的房间类型为双人房间（1V1模式）。此时无法转发媒体流。
  /// - 调用 startChannelMediaRelay 开启媒体流转发的成员角色为观众角色，仅主播角色可以转发媒体流。
  static const int channelMediaRelayPermissionDenied = 30111;

  /// 停止媒体流转发操作失败
  /// 原因通常为未开启媒体流转发。请确认调用 stopChannelMediaRelay 前，是否已成功调用 startChannelMediaRelay 开启媒体流转发。
  static const int channelMediaRelayStopFailed = 30112;

  /// 设置的媒体流加密密钥与房间中其他成员不一致，加入房间失败。
  /// 请通过 enableEncryption 重新设置加密密钥。
  static const int encryptNotSuitable = 30113;

  ///连接未找到
  static const int connectionNotFound = 30200;

  ///媒体流未找到
  static const int streamNotFound = 30201;

  ///添加track失败
  static const int addTrackFail = 30202;

  ///track未找到
  static const int trackNotFound = 30203;

  ///媒体连接已断开
  static const int mediaConnectionDisconnected = 30204;

  ///信令断开
  static const int signalDisconnected = 30205;

  ///被管理员踢出房间
  static const int serverKicked = 30206;

  ///房间被关闭
  static const int roomClosed = 30207;

  /// 因为切换频道而离开房间
  static const int leaveChannelForSwitch = 30208;
}

/// 运行时错误
class NERtcRuntimeError {
  /// 没有音频权限
  static const int admNoAuthorize = 50000;

  /// 没有视频权限
  static const int vdmNoAuthorize = 50001;

  /// 麦克风初始化失败
  static const int admRecordInitError = 50103;

  /// 麦克风打开失败
  static const int admRecordStartError = 50104;

  /// 麦克风运行错误
  static const int admRecordUnknownError = 50105;

  /// 音频播放设备初始化失败
  static const int admPlayoutInitError = 50203;

  /// 音频播放设备打开失败
  static const int admPlayoutStartError = 50204;

  /// 音频播放设备运行错误
  static const int admPlayoutUnknownError = 50205;

  /// 相机被其他应用抢占
  static const int vdmCameraDisconnectError = 50303;

  /// 相机已冻结
  static const int vdmCameraFreezedError = 50304;

  /// 相机运行错误
  static const int vdmCameraUnknownError = 50305;
}

/// 网络状态
class NERtcNetworkStatus {
  /// 未知
  static const int unknown = 0;

  /// 非常好
  static const int excellent = 1;

  /// 好
  static const int good = 2;

  /// 不太好
  static const int poor = 3;

  /// 差
  static const int bad = 4;

  /// 非常差
  static const int veryBad = 5;

  /// 无网络
  static const int down = 6;
}

/// 频道场景
class NERtcChannelProfile {
  /// 通信场景
  static const int communication = 0;

  /// 直播场景
  static const int liveBroadcasting = 1;
}

/// 远端视频流类型
class NERtcRemoteVideoStreamType {
  /// 高分辨率的远端视频流
  static const int high = 0;

  /// 低分辨率的远端视频流
  static const int low = 1;
}

/// 与会者角色， 主播/观众
class NERtcUserRole {
  /// 主播模式，能发送和接收数据
  static const int broadcaster = 0;

  /// 观众模式，只能接收数据
  static const int audience = 1;
}

/// 语音设备类型
class NERtcAudioFocusMode {
  /// 不请求音频焦点
  static const int audioFocusOff = 0;

  /// 长时间获得焦点
  static const int audioFocusGain = 1;

  /// 短暂性获得焦点，用完应立即释放，比如notification sounds
  static const int audioFocusGainTransient = 2;

  /// 临时请求, 降低其他音频应用的声音，可混音播放
  static const int audioFocusGainTransientMayDuck = 3;

  /// 短暂性获得焦点，录音或者语音识别
  static const int audioFocusGainTransientExclusive = 4;
}

/// 视频发布流类型
enum NERtcVideoSendMode {
  /// 按对端订阅格式发流
  none,

  /// 初始发送大流
  high,

  /// 初始发布小流
  low,

  /// 初始大小流同时发送
  all
}

/// 带宽受限时的视频编码降级偏好
class NERtcDegradationPreference {
  /// （默认）根据场景模式调整适应性偏好。
  ///
  /// - 通信场景中，选择 degradationBalanced 模式，在编码帧率和视频质量之间保持平衡。
  ///
  /// - 直播场景中，选择 degradationMaintainQuality 模式，降低编码帧率以保证视频质量。
  static const int degradationDefault = 0;

  /// 流畅优先，降低视频质量以保证编码帧率。在弱网环境下，降低视频清晰度以保证视频流畅，此时画质降低，画面会变得模糊，但可以保持视频流畅
  static const int degradationMaintainFrameRate = 1;

  /// 清晰优先，降低编码帧率以保证视频质量。在弱网环境下，降低视频帧率以保证视频清晰，此时可能会出现一定卡顿。
  static const int degradationMaintainQuality = 2;

  /// 在编码帧率和视频质量之间保持平衡
  static const int degradationBalanced = 3;
}

/// 视频编码帧率
class NERtcVideoFrameRate {
  /// 默认 0fps
  static const int fpsDefault = 0;

  /// 7fps
  static const int fps_7 = 7;

  /// 10fps
  static const int fps_10 = 10;

  /// 15fps
  static const int fps_15 = 15;

  ///24fps
  static const int fps_24 = 24;

  /// 30fps
  static const int fps_30 = 30;
}

/// 直播推流模式
class NERtcLiveStreamMode {
  /// 推流带视频
  static const int liveStreamModeVideo = 0;

  /// 推流纯音频
  static const int liveStreamModeAudio = 1;
}

/// 直播推流状态
class NERtcLiveStreamState {
  /// 推流中
  static const int pushing = 505;

  /// 互动直播推流失败
  static const int pushFail = 506;

  /// 推流结束
  static const int pushStopped = 511;

  /// 背景图片设置出错
  static const int imageError = 512;
}

/// LiveStream Error Code
class NERtcLiveStreamErrorCode {
  /// 成功
  static const int ok = 0;

  /// Task请求无效，被后续操作覆盖
  static const int requestIsInvaild = 1301;

  /// Task参数格式错误
  static const int invaild = 1400;

  /// 房间已经退出
  static const int roomExited = 1401;

  /// 推流任务超出上限
  static const int numLimit = 1402;

  /// 推流ID重复
  static const int duplicateId = 1403;

  /// TaskId任务不存在，或频道不存在
  static const int notFound = 1404;

  /// 请求失败
  static const int requestError = 1417;

  /// 服务器内部错误
  static const int internalServerErr = 1500;

  /// 布局参数错误
  static const int invalidLayout = 1501;

  /// 用户图片错误
  static const int userPictureError = 1502;
}

/// 直播布局
class NERtcLiveStreamLayout {
  NERtcLiveStreamLayout({
    this.width,
    this.height,
    this.backgroundImg,
    this.userTranscodingList,
    this.backgroundColor,
  });

  /// 视频推流宽度
  int? width;

  /// 视频推流高度
  int? height;

  /// 视频推流背景色 RGB
  Color? backgroundColor;

  /// 视频推流背景图
  NERtcLiveStreamImageInfo? backgroundImg;

  /// 成员布局数组
  List<NERtcLiveStreamUserTranscoding>? userTranscodingList;
}

/// 视频尺寸缩放模式
class NERtcLiveStreamVideoScaleMode {
  /// 视频尺寸等比缩放。优先保证视频内容全部显示。因视频尺寸与显示视窗尺寸不一致造成的视窗未被填满的区域填充背景色。
  static const int liveStreamModeVideoScaleFit = 0;

  /// 视频尺寸等比缩放。优先保证视窗被填满。因视频尺寸与显示视窗尺寸不一致而多出的视频将被截掉。
  static const int liveStreamModeVideoScaleCropFill = 1;
}

/// 直播成员布局
class NERtcLiveStreamUserTranscoding {
  NERtcLiveStreamUserTranscoding(
      {required this.uid,
      this.videoPush = true,
      this.audioPush = true,
      this.adaption = NERtcLiveStreamVideoScaleMode.liveStreamModeVideoScaleFit,
      this.x = 0,
      this.y = 0,
      this.width = 0,
      this.height = 0});

  /// 用户uid
  int uid;

  /// 是否推送该用户视频流，推流模式为 [NERtcLiveStreamMode.liveStreamModeAudio] 时无效
  bool videoPush;

  /// 是否推送该用户音频流
  bool audioPush;

  /// 视频流裁剪模式, 参考 [NERtcLiveStreamVideoScaleMode]
  int adaption;

  /// 画面离主画面左边距
  int x;

  /// 画面离主画面上边距
  int y;

  /// 画面在主画面的显示宽度，画面右边超出主画面会失败
  int width;

  /// 画面在主画面的显示高度，画面底边超出主画面会失败
  int height;

  Map<dynamic, dynamic> toMap() {
    final Map<dynamic, dynamic> map = <dynamic, dynamic>{};
    map['uid'] = uid;
    map['videoPush'] = videoPush;
    map['audioPush'] = audioPush;
    map['adaption'] = adaption;
    map['x'] = x;
    map['y'] = y;
    map['width'] = width;
    map['height'] = height;
    return map;
  }

  @override
  String toString() {
    return 'NERtcLiveStreamUserTranscoding{'
        'uid: $uid, videoPush: $videoPush, audioPush: $audioPush, '
        'adaption: $adaption, x: $x, y: $y, width: $width, height: $height}';
  }
}

/// 推流背景图片设置
class NERtcLiveStreamImageInfo {
  NERtcLiveStreamImageInfo(
      {this.url, this.x = 0, this.y = 0, this.width = 0, this.height = 0});

  /// 图片地址url
  String? url;

  /// 图片离主画面左边距 ， 默认 0
  int x;

  /// 图片离主画面上边距 ， 默认 0
  int y;

  /// 图片在主画面的显示宽度，图片右边超出主画面会失败 ， 默认主画面宽度
  int width;

  /// 图片在主画面的显示高度，图片底边超出主画面会失败， 默认主画面高度
  int height;

  @override
  String toString() {
    return 'NERtcLiveStreamImageInfo{'
        'url: $url, x: $x, y: $y, width: $width, height: $height}';
  }
}

/// 房间推流任务参数
class NERtcLiveStreamTaskInfo {
  NERtcLiveStreamTaskInfo(
      {required this.taskId,
      required this.url,
      this.serverRecordEnabled = false,
      this.liveMode = NERtcLiveStreamMode.liveStreamModeVideo,
      this.layout});

  /// 推流任务ID，为推流任务的唯一标识，用于过程中增删任务操作
  String taskId;

  /// 直播推流地址
  String url;

  /// 服务器录制功能是否开启
  bool serverRecordEnabled;

  /// 直播推流模式 [NERtcLiveStreamMode]
  int liveMode;

  /// 视频布局
  NERtcLiveStreamLayout? layout;

  @override
  String toString() {
    return 'NERtcLiveStreamTaskInfo{taskId:$taskId,url:$url,serverRecordEnabled:$serverRecordEnabled,layout:$layout,liveMode:$liveMode}';
  }
}

/// 频道连接状态
class NERtcConnectionState {
  /// 未知
  static const int unknown = 0;

  /// 网络连接断开
  static const int disconnected = 1;

  ///建立网络连接中
  static const int connecting = 2;

  /// 网络已连接
  static const int connected = 3;

  /// 重新建立网络连接中
  static const int reconnecting = 4;

  /// 网络连接失败
  static const int failed = 5;
}

/// 频道连接状态变更原因
class NERtcConnectionStateChangeReason {
  /// 离开房间
  static const int leaveChannel = 1;

  /// 房间被关闭
  static const int channelClosed = 2;

  /// 用户被踢
  static const int serverKicked = 3;

  /// 超时离开
  static const int timeout = 4;

  /// 加入房间
  static const int joinChannel = 5;

  /// 加入房间成功
  static const int joinSucceed = 6;

  /// 重新加入房间成功（重连）
  static const int rejoinSucceed = 7;

  /// 媒体连接断开
  static const int mediaConnectionDisconnected = 8;

  /// 信令连接断开
  static const int signalDisconnected = 9;

  /// 请求频道失败
  static const int requestChannelFailed = 10;

  /// 加入频道失败
  static const int joinChannelFailed = 11;

  ///重新分配了服务端IP
  static const int reDispatch = 12;

  ///开始使用云代理进行连接
  static const int settingProxyServer = 13;
}

/// 用户角色
class NERtcClientRole {
  /// 主播
  static const int broadcaster = 0;

  /// 观众
  static const int audience = 1;
}

/// 变声 预设值
class NERtcVoiceChangerType {
  /// 关闭
  static const int off = 0;

  /// 机器人
  static const int robot = 1;

  /// 巨人
  static const int giant = 2;

  /// 恐怖
  static const int horror = 3;

  /// 成熟
  static const int mature = 4;

  /// 男变女
  static const int manToWoman = 5;

  /// 女变男
  static const int womanToMan = 6;

  /// 男变萝莉
  static const int manToLoli = 7;

  /// 女变萝莉
  static const int womanToLoli = 8;
}

/// 美声效果
class NERtcVoiceBeautifierType {
  /// 默认关闭
  static const int off = 0;

  /// 低沉
  static const int muffled = 1;

  /// 圆润
  static const int mellow = 2;

  /// 清澈
  static const int clear = 3;

  /// 磁性
  static const int magnetic = 4;

  /// 录音棚
  static const int recordingStudio = 5;

  /// 天籁
  static const int nature = 6;

  /// KTV
  static const int ktv = 7;

  /// 悠远
  static const int remote = 8;

  /// 教堂
  static const int church = 9;

  /// 卧室
  static const int bedroom = 10;

  /// live
  static const int live = 11;
}

///伴音错误状态
class NERtcAudioMixingError {
  /// 伴音正常结束
  static const int finish = 0;

  /// 音频解码错误
  static const int errorDecode = 1;

  /// 操作中断码
  static const int errorInterrupt = 2;

  /// 404 http/https 对应的文件找不到
  static const int errorHttpNotFound = 3;

  /// 打开流/文件失败
  static const int errorOpen = 4;

  /// 获取解码信息失败/超时
  static const int errorNoInfo = 5;

  /// 无音频流
  static const int errorNoStream = 6;

  /// 无解码器
  static const int errorNoCodec = 7;

  /// 无内存
  static const int errorNoMemory = 8;

  /// 解码器打开失败/超时
  static const int errorCodecOpen = 9;

  /// 无效音频参数（声道、采样率）
  static const int errorInvalidInfo = 10;

  /// 打开流/文件超时
  static const int errorOpenTimeout = 11;

  /// 网络IO 超时
  static const int errorIOTimeout = 12;

  /// 网络IO 错误
  static const int errorIO = 13;
}

/// 媒体流优先级
class NERtcMediaPriority {
  /// 高优先级
  static const int high = 50;

  /// （默认）普通优先级
  static const int normal = 100;
}

/// 录音音质
class NERtcAudioRecordingQuality {
  /// 低音质
  static const int low = 0;

  /// （默认）中音质
  static const int medium = 1;

  /// 高音质
  static const int high = 2;
}

///录音文件所包含的内容
class NERtcAudioRecordingPosition {
  /// 录制本地和所有远端用户混音后的音频（默认）
  static const int mixedRecordingandPlayback = 0;

  /// 仅录制采集的音频
  static const int recording = 1;

  /// 仅录制播放的音频
  static const int mixedPlayback = 2;
}

///录制过程中，循环缓存的最大时间长度，单位(s)
class NERtcAudioRecordingCycleTime {
  /// 音频录制缓存时间为0，实时写文件（默认）
  static const int CycleTime0 = 0;

  /// 音频录制缓存时间为10s，StopAudioRectording()后，将缓存都写到文件，文件数据时间跨度为: [0,10s]
  static const int CycleTime10 = 10;

  /// 音频录制缓存时间为10s，StopAudioRectording()后，将缓存都写到文件，文件数据时间跨度为: [0,60s]
  static const int CycleTime60 = 60;

  /// 音频录制缓存时间为10s，StopAudioRectording()后，将缓存都写到文件，文件数据时间跨度为: [0,360s]
  static const int CycleTime360 = 360;

  /// 音频录制缓存时间为10s，StopAudioRectording()后，将缓存都写到文件，文件数据时间跨度为: [0,900s]
  static const int CycleTime900 = 900;
}

/// 音频录制配置
class NERtcAudioRecordingConfiguration {
  /// 录音文件在本地保存的绝对路径，需要精确到文件名及格式。例如：sdcard/xxx/audio.aac。请确保指定的路径存在并且可写。目前仅支持 WAV 或 AAC 文件格式
  String filePath;

  /// 录音采样率（Hz），可以设为 16000、32000（默认）、44100 或 48000
  int? sampleRate;

  /// 录音音质，只在 AAC 格式下有效。详细信息请参考[NERtcAudioRecordingQuality]
  int? quality;

  /// 录音文件所包含的内容。详细信息请参考[NERtcAudioRecordingPosition]
  int? position;

  /// 录制过程中，循环缓存的最大时间长度，单位(s)。详细信息请参考[NERtcAudioRecordingCycleTime]
  int? cycleTime;
  NERtcAudioRecordingConfiguration(
      {required this.filePath,
      this.sampleRate = 32000,
      this.quality = 1,
      this.position = 0,
      this.cycleTime = 0});

  @override
  String toString() {
    return 'NERtcAudioRecordingConfiguration{filePath:$filePath,sampleRate:$sampleRate,quality:$quality'
        'position:$position,cycleTime:$cycleTime}';
  }
}

/// 录音回调事件状态码
class NERtcAudioRecordingCode {
  /// 不支持的录音文件格式
  static const int errorSuffix = 1;

  /// 无法创建录音文件，原因通常包括：
  /// - 应用没有磁盘写入权限。
  /// - 文件路径不存在。
  static const int openFileFailed = 2;

  /// 开始录制
  static const int start = 3;

  /// 录制错误。原因通常为磁盘空间已满，无法写入
  static const int error = 4;

  /// 完成录制
  static const int finish = 5;
}

/// 媒体流转发相关的数据结构
class NERtcChannelMediaRelayInfo {
  /// 用户 ID
  int channelUid;

  /// 房间名
  String channelName;

  /// 房间 Token
  String channelToken;

  NERtcChannelMediaRelayInfo(
      {required this.channelUid,
      required this.channelName,
      required this.channelToken});

  Map<dynamic, dynamic> toMap() {
    final Map<dynamic, dynamic> map = <dynamic, dynamic>{};
    map['channelUid'] = channelUid;
    map['channelName'] = channelName;
    map['channelToken'] = channelToken;
    return map;
  }

  static NERtcChannelMediaRelayInfo fromMap(Map<String, dynamic> map) {
    return NERtcChannelMediaRelayInfo(
        channelUid: map['channelUid'] ?? 0,
        channelName: map['channelName'] ?? '',
        channelToken: map['channelToken'] ?? '');
  }

  @override
  String toString() {
    return 'NERtcChannelMediaRelayInfo{channelUid:$channelUid,channelName:$channelName,channelToken:$channelToken}';
  }
}

/// 媒体流转发参数，包括源房间、目标房间列表等
class NERtcChannelMediaRelayConfiguration {
  /// 源房间信息
  NERtcChannelMediaRelayInfo? sourceMediaInfo;

  /// 目标房间信息列表
  Map<String, NERtcChannelMediaRelayInfo> destMediaInfo;

  NERtcChannelMediaRelayConfiguration(this.sourceMediaInfo, this.destMediaInfo);
  @override
  String toString() {
    return 'NERtcChannelMediaRelayConfiguration{sourceMediaInfo:${sourceMediaInfo.toString()},destMediaInfo:$destMediaInfo}';
  }
}

/// 媒体流转发状态
class NERtcChannelMediaRelayState {
  /// 初始状态。在成功调用 [NERtcEngine.stopChannelMediaRelay] 停止跨房间媒体流转发后， [onNERtcEngineChannelMediaRelayStateDidChange] 会回调该状态
  static const int idle = 0;

  /// SDK 尝试跨房间转发媒体流
  static const int connecting = 1;

  /// 源房间主播角色成功加入目标频道
  static const int running = 2;

  /// 发生异常，详见 [onMediaRelayReceiveEvent] 的 error 中提示的错误信息
  static const int failure = 3;
}

/// 媒体流转发回调事件
class NERtcChannelMediaRelayEvent {
  /// 媒体流转发停止
  static const int disconnect = 0;

  /// 正在连接服务器，开始尝试转发媒体流
  static const int connecting = 1;

  /// 连接服务器成功
  static const int connected = 2;

  /// 视频媒体流成功转发到目标房间
  static const int videoSentSuccess = 3;

  /// 音频媒体流成功转发到目标房间
  static const int audioSentSuccess = 4;

  /// 屏幕共享等其他媒体流成功转发到目标房间
  static const int otherStreamSentSuccess = 5;

  /// 媒体流转发失败。原因包括：
  /// - reserveInvalidParameter(414)：请求参数错误。
  /// - channelMediaRelayInvalidState(30110)：重复调用 startChannelMediaRelay。
  /// - channelMediaRelayPermissionDenied(30111)：媒体流转发权限不足。例如调用 startChannelMediaRelay 的房间成员为观众角色、或房间为双人通话房间，不支持转发媒体流。
  /// - channelMediaRelayStopFailed(30112)：调用 stopChannelMediaRelay 前，未调用 startChannelMediaRelay。
  static const int failure = 100;
}

///本地发布流回退处理选项
class NERtcStreamFallbackOptions {
  /// 上行或下行网络较弱时，不对音视频流作回退处理，但不能保证音视频流的质量。
  ///
  /// 该选项只对 [NERtcEngine.setLocalPublishFallbackOption] 方法有效，对 [NERtcEngine.setRemoteSubscribeFallbackOption] 方法无效。
  static const int disabled = 0;

  /// 在下行网络条件较差的情况下，SDK 将只接收视频小流，即低分辨率、低码率视频流。
  ///
  /// 该选项只对 [setRemoteSubscribeFallbackOption] 方法有效，对 [setLocalPublishFallbackOption] 方法无效。
  static const int videoStreamLow = 1;

  /// 上行网络较弱时，只发布音频流。
  /// 下行网络较弱时，先尝试只接收视频小流，即低分辨率、低码率视频流。如果网络环境无法显示视频，则再回退到只接收音频流。
  static const int audioOnly = 2;
}

/// 配置媒体流加密模式和密钥
class NERtcEncryptionConfig {
  /// 媒体流加密模式
  NERtcEncryptionMode mode;

  /// 媒体流加密密钥。字符串类型，推荐设置为英文字符串
  String key;

  NERtcEncryptionConfig(this.mode, this.key);

  @override
  String toString() {
    return 'NERtcEncryptionConfig{mode:$mode,key:$key}';
  }
}

/// 媒体流加密模式
enum NERtcEncryptionMode {
  ///128 位 SM4 加密，ECB 模式。
  GMCryptoSM4ECB
}

/// nertc 私有化服务器地址配置
class NERtcServerAddresses {
  /// 获取通道信息服务器。
  String? channelServer;

  /// 统计上报服务器
  String? statisticsServer;

  /// roomServer服务器
  String? roomServer;

  /// 兼容性配置服务器
  String? compatServer;

  /// nos 域名解析服务器
  String? nosLbsServer;

  /// 默认nos 上传服务器
  String? nosUploadSever;

  /// 获取NOS token 服务器
  String? nosTokenServer;

  /// 美颜鉴权服务 (安卓独有)
  String? sdkConfigServer;

  /// 云代理服务器
  String? cloudProxyServer;

  /// webSocket 信令代理地址
  String? webSocketProxyServer;

  /// quic 信令代理地址
  String? quicProxyServer;

  /// 媒体代理地址
  String? mediaProxyServer;

  /// 统计调度服务器
  String? statisticsDispatchServer;

  /// 统计备份服务器
  String? statisticsBackupServer;

  /// 是否使用IPv6（默认false）
  bool? useIPv6;

  NERtcServerAddresses({
    this.channelServer,
    this.statisticsServer,
    this.roomServer,
    this.compatServer,
    this.nosLbsServer,
    this.nosUploadSever,
    this.nosTokenServer,
    this.useIPv6 = false,
    this.sdkConfigServer,
    this.cloudProxyServer,
    this.webSocketProxyServer,
    this.quicProxyServer,
    this.mediaProxyServer,
    this.statisticsDispatchServer,
    this.statisticsBackupServer,
  });

  NERtcServerAddresses.fromMap(Map map) {
    channelServer = map['channelServer'] as String?;
    statisticsServer = map['statisticsServer'] as String?;
    roomServer = map['roomServer'] as String?;
    compatServer = map['compatServer'] as String?;
    nosLbsServer = map['nosLbsServer'] as String?;
    nosUploadSever = map['nosUploadSever'] as String?;
    nosTokenServer = map['nosTokenServer'] as String?;
    useIPv6 = (map['useIPv6'] ?? false) as bool;
    sdkConfigServer = map['sdkConfigServer'] as String?;
    cloudProxyServer = map['cloudProxyServer'] as String?;
    webSocketProxyServer = map['webSocketProxyServer'] as String?;
    quicProxyServer = map['quicProxyServer'] as String?;
    mediaProxyServer = map['mediaProxyServer'] as String?;
    statisticsDispatchServer = map['statisticsDispatchServer'] as String?;
    statisticsBackupServer = map['statisticsBackupServer'] as String?;
  }

  @override
  String toString() {
    return 'NERtcServerAddresses{'
        'channelServer: $channelServer, '
        'statisticsServer: $statisticsServer, '
        'roomServer: $roomServer, '
        'compatServer: $compatServer, '
        'nosLbsServer: $nosLbsServer, '
        'nosUploadSever: $nosUploadSever, '
        'nosTokenServer: $nosTokenServer, '
        'useIPv6: $useIPv6, '
        'sdkConfigServer: $sdkConfigServer, '
        'cloudProxyServer: $cloudProxyServer, '
        'webSocketProxyServer: $webSocketProxyServer, '
        'quicProxyServer: $quicProxyServer, '
        'mediaProxyServer: $mediaProxyServer, '
        'statisticsDispatchServer: $statisticsDispatchServer, '
        'statisticsBackupServer: $statisticsBackupServer'
        '}';
  }
}

///
/// 音频会话控制权限。
/// 控制SDK 对 Audio Session 的控制权限, 仅**iOS**平台有效。
///
enum NERtcAudioSessionOperationRestriction {
  /// 没有限制，SDK 可以完全控制 Audio Session 操作。
  none,

  /// 限制 SDK 对 Audio Session 进行任何操作，SDK 将不能再对 Audio Session 进行任何配置。
  all,

  ///离开房间时，SDK 会保持 Audio Session 处于活动状态。
  deactivateSession,
}

///视频水印状态
class NERtcLocalVideoWatermarkState {
  ///视频水印设置成功
  static const int setSuccess = 0;

  ///设备不支持
  static const int deviceNotSupported = 1;

  ///图片格式不支持
  static const int imgFormatNotSupported = 2;

  ///图片数量设置错误
  static const int imgNumErr = 3;

  ///图片尺寸设置错误
  static const int imgSizeErr = 4;

  ///图片帧率设置错误，帧率的取值范围是 0~30.
  static const int imgFPSErr = 5;

  ///字体设置错误
  static const int fontErr = 6;

  ///窗口透明度设置错误
  static const int wmAlphaErr = 7;

  ///文本内容为空
  static const int textContentEmpty = 8;

  ///取消水印
  static const int cancel = 20;
}

///混响参数
class NERtcReverbParam {
  ///湿信号，取值范围为 0 ~ 1
  double wetGain;

  ///干信号，取值范围为 0 ~ 1
  double dryGain;

  ///混响阻尼，取值范围为 0 ~ 1
  double damping;

  ///房间大小，取值范围为 0.1 ~ 2
  double roomSize;

  ///持续强度（余响），取值范围为 0.1 ~ 20
  double decayTime;

  ///延迟长度，取值范围为 0 ~ 1
  double preDelay;

  NERtcReverbParam(
      {this.damping = 1.0,
      this.decayTime = 0.1,
      this.dryGain = 1.0,
      this.wetGain = 0.0,
      this.roomSize = 0.1,
      this.preDelay = 0.0});

  @override
  String toString() {
    return 'NERtcReverbParam{damping:$damping,decayTime:$decayTime,dryGain:$dryGain,'
        'wetGain:$wetGain,roomSize:$roomSize,preDelay:$preDelay}';
  }
}

///Last mile 网络探测配置
class LastmileProbeConfig {
  ///是否探测上行网络。 不发流的用户，例如直播房间中的普通观众，无需进行上行网络探测-true: 探测,false: 不探测。
  bool probeUplink;

  ///是否探测下行网络 true: 探测, false: 不探测。
  bool probeDownlink;

  ///本端期望的最高发送码率。 单位为 bps，范围为 [100000, 5000000]。 推荐参考 [setLocalVideoConfig]中的码率值设置该参数的值
  int expectedUplinkBitrate;

  ///本端期望的最高接收码率。 单位为 bps，范围为 [100000, 5000000]
  int expectedDownlinkBitrate;

  LastmileProbeConfig(this.probeUplink, this.probeDownlink,
      this.expectedUplinkBitrate, this.expectedDownlinkBitrate);

  @override
  String toString() {
    return 'LastmileProbeConfig{probeUplink: $probeUplink,probeDownlink: $probeDownlink,'
        'expectedUplinkBitrate: $expectedUplinkBitrate,expectedDownlinkBitrate: $expectedDownlinkBitrate}';
  }
}

///视频图像畸变矫正相关设置
class NERtcVideoCorrectionConfiguration {
  /// 矫正区域的左上顶点
  CGPoint topLeft;

  /// 矫正区域的右上顶点
  CGPoint topRight;

  /// 矫正区域的左下顶点
  CGPoint bottomLeft;

  ///矫正区域的右下顶点
  CGPoint bottomRight;

  ///画布宽度
  double? canvasWidth;

  ///画布高度
  double? canvasHeight;

  ///是否镜像显示
  bool? enableMirror;

  NERtcVideoCorrectionConfiguration(
      {required this.topLeft,
      required this.topRight,
      required this.bottomLeft,
      required this.bottomRight,
      this.canvasHeight = 0,
      this.canvasWidth = 0,
      this.enableMirror = false});

  @override
  String toString() {
    return 'NERtcVideoCorrectionConfiguration{topLeft: $topLeft,topRight: $topRight,bottomLeft: $bottomLeft,'
        'bottomRight: $bottomRight,canvasHeight: $canvasHeight,canvasWidth: $canvasWidth, enableMirror:$enableMirror}';
  }
}

/// 自定义背景图片的类型
class VirtualBackgroundSourceType {
  /// （默认）背景图像为纯色
  static const int color = 1;

  /// 背景图像只支持 PNG 或 JPG 格式的文件
  static const int image = 2;

  /// 背景图为虚化处理后的背景图
  static const int blur = 3;
}

/// 自定义背景图的虚化程度
class BlurDegree {
  /// 自定义背景图的虚化程度为低。用户差不多能看清背景。
  static const int low = 1;

  /// 自定义背景图的虚化程度为中。用户较难看清背景。
  static const int medium = 2;

  /// 默认）自定义背景图的虚化程度为高。用户很难看清背景。
  static const int high = 3;
}

///自定义背景图像
class NERtcVirtualBackgroundSource {
  ///自定义背景图片的类型。 请参阅[VirtualBackgroundSourceType]
  int backgroundSourceType;

  ///自定义背景图像的颜色。格式为 RGB 定义的十六进制整数，不带 # 号。
  ///例如 0xFFB6C1 代表浅粉色。默认值为 0xFFFFFF，表示白色。取值范围是 [0x000000,0xFFFFFF]。如果该值无效，SDK 将原始背景图片替换为白色的图片
  int color;

  ///自定义背景图片的本地绝对路径。支持 PNG 和 JPG 格式
  String source;

  ///自定义背景图的虚化程度,请参阅[BlurDegree]
  int blur_degree;
  NERtcVirtualBackgroundSource(
      {required this.backgroundSourceType,
      required this.color,
      this.source = '',
      required this.blur_degree});

  @override
  String toString() {
    return 'NERtcVirtualBackgroundSource{backgroundSourceType: $backgroundSourceType,color: $color,'
        ',source: $source, blur_degree: $blur_degree}';
  }
}

///云代理模式
class NERtcTransportType {
  ///关闭已设置的云代理
  static const int NERtcTransportTypeNoneProxy = 0;

  ///开启 UDP 协议的云代理
  static const int NERtcTransportTypeUDPProxy = 1;
}

///美颜类型
class NERtcBeautyEffectType {
  ///未知
  static const int kNERtcBeautyUnknownType = -1;

  ///美牙。强度默认值为 0.0。
  static const int kNERtcBeautyWhiteTeeth = 0;

  ///亮眼。强度默认值为 0.0
  static const int kNERtcBeautyLightEye = 1;

  ///美白。强度默认值为 0.0
  static const int kNERtcBeautyWhiten = 2;

  ///磨皮。强度默认值为 0.0
  static const int kNERtcBeautySmooth = 3;

  ///小鼻。强度默认值为 0.0
  static const int kNERtcBeautySmallNose = 4;

  ///眼距调整。强度默认值为 0.5
  static const int kNERtcBeautyEyeDis = 5;

  ///眼角调整。强度默认值为 0.5
  static const int kNERtcBeautyEyeAngle = 6;

  ///嘴型调整。强度默认值为 0.5
  static const int kNERtcBeautyMouth = 7;

  ///大眼。强度默认值为 0.0
  static const int kNERtcBeautyBigEye = 8;

  ///小脸。强度默认值为 0.0
  static const int kNERtcBeautySmallFace = 9;

  ///下巴调整。强度默认值为 0.0
  static const int kNERtcBeautyJaw = 10;

  ///瘦脸。强度默认值为 0.0
  static const int kNERtcBeautyThinFace = 11;

  ///红润。强度默认值为 0.0
  static const int kNERtcBeautyFaceRuddy = 12;

  ///长鼻。强度默认值为 0.5
  static const int kNERtcBeautyLongNose = 13;

  ///人中调整。强度默认值为 0.5
  static const int kNERtcBeautyPhiltrum = 14;

  ///嘴角调整。强度默认值为 0.5
  static const int kNERtcBeautyMouthAngle = 15;

  ///圆眼。强度默认值为 0.0
  static const int kNERtcBeautyRoundEye = 16;

  ///开眼角。强度默认值为 0.0
  static const int kNERtcBeautyEyeCorner = 17;

  ///V脸。强度默认值为 0.0
  static const int kNERtcBeautyVFace = 18;

  ///瘦下颌。强度默认值为 0.0
  static const int kNERtcBeautyUnderJaw = 19;

  ///窄脸。强度默认值为 0.0
  static const int kNERtcBeautyNarrowFace = 20;

  ///颧骨调整。强度默认值为 0.0
  static const int kNERtcBeautyCheekBone = 21;

  ///锐化调整。强度默认值为 0.0
  static const int kNERtcBeautyFaceSharpen = 22;

  ///调整嘴巴宽度。强度默认值为 0.5
  static const int kNERtcBeautyMouthWider = 23;

  ///袪抬头纹。强度默认值为 0.0
  static const int kNERtcBeautyForeheadWrinkles = 24;

  ///袪黑眼圈。强度默认值为 0.0
  static const int kNERtcBeautyDarkCircles = 25;

  ///袪法令纹。强度默认值为 0.0
  static const int kNERtcBeautySmileLines = 26;

  ///短脸。强度默认值为 0.0
  static const int kNERtcBeautyShortFace = 27;
}

/// 语音音量
class NERtcAudioVolumeInfo {
  /// 用户 ID
  int? uid;

  /// 音量[0-100]
  int? volume;

  ///远端用户音频辅流的音量，取值范围为 [0,100]。 如果 subStreamVolume 为 0，表示该用户未发布音频辅流或音频辅流没有音量
  int? subStreamVolume;

  NERtcAudioVolumeInfo(this.uid, this.volume, this.subStreamVolume);
  @override
  String toString() {
    return 'AudioVolumeInfo{uid: $uid, volume: $volume, subStreamVolume: $subStreamVolume}';
  }
}

///音频流类型
class NERtcAudioStreamType {
  ///音频流主流
  static const int kNERtcAudioStreamTypeMain = 0;

  ///音频流辅流
  static const int kNERtcAudioStreamTypeSub = 1;
}

///网络质量类型
class NERtcNetworkQuality {
  ///网络质量未知
  static const int kNERtcNetworkQualityUnknown = 0;

  ///网络质量极好
  static const int kNERtcNetworkQualityExcellent = 1;

  ///用户主观感觉和 excellent 差不多，但码率可能略低于 excellent
  static const int kNERtcNetworkQualityGood = 2;

  ///用户主观感受有瑕疵但不影响沟通
  static const int kNERtcNetworkQualityPoor = 3;

  ///勉强能沟通但不顺畅
  static const int kNERtcNetworkQualityBad = 4;

  ///网络质量非常差，基本不能沟通
  static const int kNERtcNetworkQualityVeryBad = 5;

  ///完全无法沟通
  static const int kNERtcNetworkQualityDown = 6;
}

///Last mile 质量探测结果的状态
class NERtcLastmileProbeResultState {
  ///表示本次 last mile 质量探测的结果是完整的
  static const int kNERtcLastmileProbeResultComplete = 0;

  ///表示本次 last mile 质量探测未进行带宽预测，因此结果不完整。一个可能的原因是测试资源暂时受限
  static const int kNERtcLastmileProbeResultIncompleteNoBwe = 1;

  ///未进行 last mile 质量探测。一个可能的原因是网络连接中断
  static const int kNERtcLastmileProbeResultUnavailable = 2;
}

///媒体 pub 类型
class NERtcMediaPubType {
  ///音频 pub 类型
  static const int kNERtcMediaPubTypeAudio = 0;
}

///视频图片水印配置
class NERtcVideoWatermarkImageConfig {
  ///整体水印透明度，取值范围 为 0.0 ~ 1.0，默认值为 1.0，表示不透明
  double? wmAlpha;

  ///水印框的宽度，单位为像素，默认值为 0，表示按原始图宽
  int? wmWidth;

  ///水印框的高度，单位为像素，默认值为 0，表示按原始图高
  int? wmHeight;

  ///水平左上角与视频图像左上角的水平距离，单位为像素，默认值为 0
  int? offsetX;

  ///水平左上角与视频图像左上角的垂直距离，单位为像素，默认值为 0
  int? offsetY;

  ///图片绝对路径，支持多个图片路径
  List<String?> imagePaths;

  ///播放帧率，默认为 0 表示不自动切换图片，图片单帧静态展示。 帧率最高不超过 30fps，如果设置帧率高于视频流帧率，则按照视频流帧率展示
  int? fps;

  ///是否设置循环，默认为 true，设置为 false 时图像组播放完毕后水印消失
  bool? loop;

  NERtcVideoWatermarkImageConfig(
      {this.wmAlpha = 1.0,
      this.wmWidth = 0,
      this.wmHeight = 0,
      this.offsetX = 0,
      this.offsetY = 0,
      required this.imagePaths,
      this.fps = 0,
      this.loop = true});

  @override
  String toString() {
    return 'NERtcVideoWatermarkImageConfig{'
        'wmAlpha: $wmAlpha, wmWidth: $wmWidth, wmHeight: $wmHeight, '
        'offsetX: $offsetX, offsetY: $offsetY, imagePaths: $imagePaths, fps: $fps, loop: $loop}';
  }
}

/// 视频文字水印配置
class NERtcVideoWatermarkTextConfig {
  ///整体水印透明度，取值范围 为 0.0 ~ 1.0，默认值为 1.0，表示不透明
  double? wmAlpha;

  ///水印框的宽度，单位为像素，默认值为 0，表示按原始图宽
  int? wmWidth;

  ///水印框的高度，单位为像素，默认值为 0，表示按原始图高
  int? wmHeight;

  ///水平左上角与视频图像左上角的水平距离，单位为像素，默认值为 0
  int? offsetX;

  ///水平左上角与视频图像左上角的垂直距离，单位为像素，默认值为 0
  int? offsetY;

  ///水印框内背景颜色。ARGB 格式，默认值为 0x88888888，即灰色。 支持透明度设置
  int? wmColor;

  ///字体大小。默认值为 15，单位为像素(px)
  int? fontSize;

  ///字体颜色。ARGB 格式。默认值为 0xFFFFFFFF，即白色
  int? fontColor;

  ///字体文件绝对路径
  String? fontNameOrPath;

  ///文字内容，设置为空时，表示不添加文字水印
  String? content;

  NERtcVideoWatermarkTextConfig(
      {this.wmAlpha = 1.0,
      this.wmWidth = 0,
      this.wmHeight = 0,
      this.offsetX = 0,
      this.offsetY = 0,
      this.fontNameOrPath = '',
      this.fontColor = 0xFFFFFFFF,
      this.fontSize = 15,
      this.wmColor = 0x88888888,
      required this.content});

  @override
  String toString() {
    return 'NERtcVideoWatermarkTextConfig{'
        'wmAlpha: $wmAlpha, wmWidth: $wmWidth, wmHeight: $wmHeight, '
        'offsetX: $offsetX, offsetY: $offsetY, wmColor: $wmColor, '
        'fontSize: $fontSize, fontColor: $fontColor, fontNameOrPath: $fontNameOrPath, '
        'content: $content}';
  }
}

/// 时间戳水印配置
class NERtcVideoWatermarkTimestampConfig {
  ///整体水印透明度，取值范围 为 0.0 ~ 1.0，默认值为 1.0，表示不透明
  double? wmAlpha;

  ///水印框的宽度，单位为像素，默认值为 0，表示按原始图宽
  int? wmWidth;

  ///水印框的高度，单位为像素，默认值为 0，表示按原始图高
  int? wmHeight;

  ///水平左上角与视频图像左上角的水平距离，单位为像素，默认值为 0
  int? offsetX;

  ///水平左上角与视频图像左上角的垂直距离，单位为像素，默认值为 0
  int? offsetY;

  ///水印框内背景颜色。ARGB 格式，默认值为 0x88888888，即灰色。 支持透明度设置
  int? wmColor;

  ///字体大小。默认值为 15，单位为像素(px)
  int? fontSize;

  ///字体颜色。ARGB 格式。默认值为 0xFFFFFFFF，即白色
  int? fontColor;

  ///字体文件绝对路径
  String? fontNameOrPath;

  NERtcVideoWatermarkTimestampConfig(
      {this.wmAlpha = 1.0,
      this.wmWidth = 0,
      this.wmHeight = 0,
      this.offsetX = 0,
      this.offsetY = 0,
      this.fontNameOrPath = '',
      this.fontColor = 0xFFFFFFFF,
      this.fontSize = 15,
      this.wmColor = 0x88888888});

  @override
  String toString() {
    return 'NERtcVideoWatermarkTimestampConfig{'
        'wmAlpha: $wmAlpha, wmWidth: $wmWidth, wmHeight: $wmHeight, '
        'offsetX: $offsetX,offsetY: $offsetY, wmColor: $wmColor, '
        'fontSize: $fontSize, fontColor: $fontColor, fontNameOrPath: $fontNameOrPath}';
  }
}

///视频水印设置，目前支持三种类型的水印，但只能其中选择一种水印生效
class NERtcVideoWatermarkConfig {
  /// 视频水印类型
  NERtcVideoWatermarkType WatermarkType;

  /// 图片水印配置，watermarkType = kNERtcWatermarkTypeImage 时生效
  NERtcVideoWatermarkImageConfig? imageWatermark;

  /// 文字水印配置，watermarkType = kNERtcWatermarkTypeText 时生效
  NERtcVideoWatermarkTextConfig? textWatermark;

  /// 时间戳水印配置，watermarkType = kNERtcWatermarkTypeTimestamp 时生效
  NERtcVideoWatermarkTimestampConfig? timestampWatermark;

  NERtcVideoWatermarkConfig(
      {required this.WatermarkType,
      this.imageWatermark,
      this.textWatermark,
      this.timestampWatermark});
  @override
  String toString() {
    return 'NERtcVideoWatermarkConfig{'
        'WatermarkType: ${WatermarkType.toString()}, '
        'imageWatermark: ${imageWatermark.toString()}, '
        'textWatermark: ${textWatermark.toString()}, '
        'timestampWatermark: ${timestampWatermark.toString()}'
        '}';
  }
}

///视频帧格式
class NERtcVideoFrameFormat {
  //I420 视频格式。
  static const int I420 = 0;
  //iOS NV12 视频格式。
  static const int NV12 = 1;
  //iOS BGRA 视频格式。
  static const int BGRA = 2;
  //Android NV21 视频格式。
  static const int NV21 = 3;
  //Android RGBA 视频格式。
  static const int RGBA = 4;
  //Android TEXTURE_OES 视频格式。
  static const int TEXTURE_OES = 5;
  //Android TEXTURE_RGB 视频格式。
  static const int TEXTURE_RGB = 6;
}

/// 视频数据的旋转方向
class NERtcVideoRotationType {
  //0度
  static const int kNERtcVideoRotation_0 = 0;
  //90度
  static const int kNERtcVideoRotation_90 = 90;
  //180度
  static const int kNERtcVideoRotation_180 = 180;
  //270度
  static const int kNERtcVideoRotation_270 = 270;
}

/// 视频帧信息
class NERtcVideoFrame {
  /// 视频高，即视频帧在横轴上的像素
  int width;

  /// 视频宽，即视频帧在纵轴上的像素
  int height;

  /// 视频顺时针旋转角度。详细信息请参考[NERtcVideoRotationType]
  int rotation;

  /// 视频帧格式，详细信息请参考[NERtcVideoFrameFormat]
  int format;

  /// 视频时间戳，单位为毫秒
  int timeStamp;

  /// 视频数据
  Uint8List data;

  /// Y分量存储步长
  int? strideY;

  /// U分量存储步长
  int? strideU;

  /// V分量存储步长
  int? strideV;

  /// 外部视频输入时，相机采集的 OES 纹理 ID
  int? textureId;

  /// 外部视频输入时的纹理矩阵
  List<double>? transformMatrix;

  NERtcVideoFrame(
      {required this.width,
      required this.height,
      required this.rotation,
      required this.format,
      required this.timeStamp,
      required this.data,
      this.strideY,
      this.strideU,
      this.strideV,
      this.textureId,
      this.transformMatrix});
  @override
  String toString() {
    return 'NERtcVideoFrame{width: $width, height: $height, rotation: $rotation,'
        'format: $format, timeStamp: $timeStamp, strideY: $strideY,'
        'strideU: $strideU, strideV: $strideV, textureId: $textureId,'
        'transformMatrix: $transformMatrix}';
  }
}
